PRO cor_proj_cod,coord,proj,ctype1,ctype2,status

;+
; NAME:
;    cor_proj_cod
; CATEGORY:
;    JPBLib Coordinates conversions
; PURPOSE:
;    converts coordinates and projection types into a valid fits ctype
; CALLING SEQUENCE:
;    cor_proj_cod,coord,proj,ctype1,ctype2,status
; INPUT:
;    coord        : Type of ccordinates
;    proj         : type of projection
; OUTPUT:
;    ctype1       : fits ctype1 sting
;    ctype2       : fits ctype2 sting
;    status       : status =0 if OK
; OPTIONAL INPUT:
;    None
; INPUT KEYWORDS:
;    None
; OUTPUT KEYWORDS:
;    None
; SIDE EFFECTS:
;    None
; EXAMPLES:
;    cor_proj_cod,'DEC','TAN',ctype1,ctype2,status
;    IDL> print,ctype1,ctype2
;    DEC--TAN
;    RA---TAN
; RESTRICTIONS:
;    None
; PROCEDURES CALLED:
;    None
; REVISION HISTORY:
;    Created by Bernard Jean-Philippe Jan 31 1995
;-

routine='cor_proj_cod'

IF N_params() EQ 0 THEN BEGIN
  auto_help,routine
  goto,sortie
ENDIF

ccoord='?' & pproj='?'
message,proj+' '+coord,/info

IF proj EQ 'MOL' THEN pproj='MOL'
IF proj EQ 'SIN' THEN pproj='SIN'
IF proj EQ 'TAN' THEN pproj='TAN'
IF proj EQ 'STG' THEN pproj='STG'
IF proj EQ 'GLS' THEN pproj='GLS'
IF proj EQ 'AIT' THEN pproj='AIT'
IF proj EQ 'MER' THEN pproj='MER'
IF proj EQ 'CAR' THEN pproj='CAR'
IF proj EQ 'CSC' THEN pproj='CSC'

IF coord EQ 'ELON' OR coord EQ 'ELAT' THEN BEGIN
  ccoord='ELON'
  ccoord1='ELAT'
ENDIF
IF coord EQ 'GLON' OR coord EQ 'GLAT' THEN BEGIN
 ccoord='GLON'
 ccoord1='GLAT'
ENDIF
IF coord EQ 'RA' OR coord EQ 'DEC' THEN BEGIN
  ccoord='RA'
  ccoord1='DEC'
ENDIF

status=0
IF ccoord EQ '?' THEN BEGIN
  print,coord,' Unrecognized coordinate system'
  status=-1
  goto,sortie
ENDIF
IF pproj EQ '?' THEN BEGIN
  print,proj,' Unrecognized projection'
  status=-1
  goto,sortie
ENDIF

strmoins='--------'
st1=strmid(strmoins,0,8-(strlen(ccoord)+strlen(pproj)))
ctype1=ccoord+st1+pproj
st2=strmid(strmoins,0,8-(strlen(ccoord1)+strlen(pproj)))
ctype2=ccoord1+st2+pproj

sortie:
END
