PRO cor_proj_dec,ctype,coord,proj,projtype,status,silent=silent

;+
; NAME:
;	    cor_proj_dec
; CATEGORY:
;    JPBLib Coordinates conversions
; PURPOSE:
;        decodes ctype keyword value into coordinate and projection type
; CALLING SEQUENCE: 
;	   cor_proj_dec,ctype,coord,proj,projtype,status,silent=silent
; INPUTS:
;       ctype = ctype value (e.g. 'RA---TAN')
; OPTIONAL INPUT PARAMETERS:
; OUTPUTS:
;	  coord    = coordinate system (e.g. 'RA')
;	  proj     = projection (e.g. 'TAN')
;      projtype = projection type (e.g. 'ZEN')
;      status   = 1 if OK, 0 otherwise
; OPTIONAL OUTPUT PARAMETERS:
; ACCEPTED KEY-WORDS:
;       silent   = If set keeps silent
; COMMON BLOCKS:
; SIDE EFFECTS:
; RESTRICTIONS:
; PROCEDURE:
;	Straightforward.
;    Uses : remchar
; MODIFICATION HISTORY:
;	Written Jan 31 1995 J.P. Bernard, IPAC.
;-

routine='cor_proj_dec'

IF N_params() EQ 0 THEN BEGIN
  auto_help,routine
  goto,sortie
ENDIF

pos=strpos(ctype,'-')
IF pos EQ -1 THEN BEGIN
  print,'No - found'
  status=-1
  goto,sortie
ENDIF

;Decode coord and proj
ccoord=strmid(ctype,0,pos)
;remchar,ccoord,'-'
pproj=strmid(ctype,pos+1,strlen(ctype)-pos)
remchar,pproj,'-'
coord='?' & proj='?'
IF not keyword_set(silent) THEN print,pproj,' ',ccoord

IF ccoord EQ 'ELON' OR ccoord EQ 'ELAT' THEN coord='ELON'
IF ccoord EQ 'GLON' OR ccoord EQ 'GLAT' THEN coord='GLON'
IF ccoord EQ 'RA' OR ccoord EQ 'DEC' THEN coord='RA'

zen=['AZP','TAN','SIN','STG','ARC','ZPN','ZEA','AIR']
cyl=['CYP','CAR','MER','CEA']
con=['COP','COD','COE','COO']
cov=['BON','PCO','GLS','PAR','AIT','MOL','CSC','QSC','TSC']
ind=where(zen EQ pproj,count)
IF count NE 0 THEN BEGIN
  proj=zen(ind)
  projtype='ZEN'
ENDIF
ind=where(cyl EQ pproj,count)
IF count NE 0 THEN BEGIN
  proj=cyl(ind)
  projtype='CYL'
ENDIF
ind=where(con EQ pproj,count)
IF count NE 0 THEN BEGIN
  proj=con(ind)
  projtype='CON'
ENDIF
ind=where(cov EQ pproj,count)
IF count NE 0 THEN BEGIN
  proj=cov(ind)
  projtype='COV'
ENDIF
proj=proj(0)

status=0
IF coord EQ '?' THEN BEGIN
  print,ccoord,' Unrecognized coordinate system'
  status=-1
  goto,sortie
ENDIF
IF proj EQ '?' THEN BEGIN
  print,pproj,' Unrecognized projection'
  status=-1
  goto,sortie
ENDIF
sortie:
END
