PRO make_gsfc_h,h,FBB=FBB,allsky=allsky,coord=coord,proj=proj, $
                sixpack=sixpack,tee=tee,reso=reso

;+
; NAME:
;       make_gsfc_h
; CALLING SEQUENCE:
;       make_gsfc_h,h
; PURPOSE:
;	This is to create a header for the 1024*512 images created at GSFC
;	Restricted to the first realease zone: 213:298 (|b|<=15)
; INPUTS:
; OPTIONAL INPUT:
;	if FBB if set a header for the reduced image (1024,61) is done
;       allsky = make a all-sky header
;	coord = coordinate system
;	proj  = projection type
;	sixpack = makes a 2000 eclipic sixpack cube header
;	tee     = makes a 2000 eclipic unfolded Tee header
;	righttee     = makes a 2000 eclipic unfolded Right-Tee header
;	reso    = resolution of the cube header (default=9)
; OUTPUTS:
;       h = header to be created
; PROCEDURE AND SUBROUTINE USED
; 	mkhdr,write_header
; SIDE EFFECTS:
;      none
; MODIFICATION HISTORY:
;       written JPB Mon Nov 29 1993 @ IPAC
;       modif JPB Thu Feb 17 1994 @ IPAC to include /FBB and new ctype1 v1.1
;	modif JPB Fri Jan 27 1995 @ IPAC to put center at proj center
;          added coord and proj keywords
;	   changed cdelt, added bunit
;       modif JPB 28/05/02 @ IAS to sparate /tee from /righttee. Added cubetype to keywords
;-

IF N_PARAMS(0) LT 1 THEN begin
  print,'Calling Sequence: make_gsfc_h,h'
  print,'Accepted Key-words: [,/allsky][,/FBB][,proj=][,coord=][/sixpack]'
  print,'                    [,/tee][,/righttee][,reso=]'
  goto,sortie
ENDIF

ver='1.3'

IF not keyword_set(reso) THEN reso=9

bunit='MJy/sr'
cubetype='unknown'

IF keyword_set(sixpack) OR keyword_set(tee) OR keyword_set(righttee) THEN BEGIN
  Nface=2^(reso-1)
  IF keyword_set(sixpack) THEN BEGIN
    Nx=3*Nface & Ny=2*Nface
    crpix1=(1.*Nx-1.)/2.+1
    crpix2=(1.*Ny-1.)/2.+1
    cubetype='SIXPACK'
  ENDIF
  IF keyword_set(righttee) THEN BEGIN
    Nx=4*Nface & Ny=3*Nface
    crpix1=(1.*Nx-1.)/2.+1
    crpix2=(1.*Ny-1.)/2.+1
    cubetype='RIGHTTEE'
  ENDIF
  IF keyword_set(tee) THEN BEGIN
    Nx=4*Nface & Ny=3*Nface
    crpix1=5.*Nx/8.+0.5
    crpix2=(1.*Ny-1.)/2.+1
    cubetype='TEE'
  ENDIF
  proj='CSC' & coord='ELON'
  Nmin=0
  Nmax=511
  cdelt1=-360./(1.*Nx)
  cdelt2=-cdelt1
  crval1=0.
  crota1=0.
  crval2=0.
  equ=2000
ENDIF ELSE BEGIN
  IF not keyword_set(proj) THEN proj='GLS'	;default: Global Sinusoidal
  IF not keyword_set(coord) THEN coord='GLON'	;default: Galactic
  Nx=1024
  Ny=512
  if keyword_set(allsky) THEN  BEGIN
    Nmin=0
    Nmax=511
  ENDIF ELSE BEGIN
    Nmin=213
    Nmax=298
    Ny=86
  ENDELSE
  cdelt1=-360./(1.*Nx)
  crpix1=512.5			;(1024-1)/2+1
  crval1=0.
  crota1=0.
  bmin=Nmin/511.*180.-90.
  bmax=Nmax/511.*180.-90.
  cdelt2=(bmax-bmin)/(1.*Ny)
  crpix2=256.5			;(512-1)/2+1
  crval2=1.e-15
  equ=1950			;??? not sure
ENDELSE
cor_proj_cod,coord,proj,ctype1,ctype2,status

IF status NE 0 THEN BEGIN
  print,'Unrecognized projection/coord'
  goto,sortie
ENDIF


type=4			;REAL*4
mkhdr,h,type,[Nx,Ny]

crota2=0.
bzero=0.
bscale=1.
bitpix=16		;store as integers

sxaddpar,h,'BITPIX',bitpix
sxaddpar,h,'BSCALE',bscale
sxaddpar,h,'BZERO',bzero
sxaddpar,h,'BUNIT',bunit
sxaddpar,h,'CDELT1',cdelt1
sxaddpar,h,'CRPIX1',crpix1
sxaddpar,h,'CRVAL1',crval1
sxaddpar,h,'CTYPE1',ctype1
sxaddpar,h,'CDELT2',cdelt2
sxaddpar,h,'CRPIX2',crpix2
sxaddpar,h,'CRVAL2',crval2
sxaddpar,h,'CROTA2',crota2
sxaddpar,h,'CTYPE2',ctype2
;sxaddpar,h,'LONGPOLE',180.
;sxaddpar,h,'LONGPOLE',-90.
sxaddpar,h,'LONPOLE',-90.
sxaddpar,h,'EQUINOX',equ
sxaddpar,h,'CUBETYPE',cubetype

comment='written by make_gsfc_h.pro V'+ver
sxaddpar,h,'HISTORY',comment
comment='on '+systime(0)
sxaddpar,h,'HISTORY',comment

sortie:
END
