pro precess_array, ra, dec, equinox1, equinox2, PRINT = print

;+
; NAME:
;    precess_array
; CATEGORY:
;    Coordinate conversions
; PURPOSE:
;    precess cooridnate arrays
; CALLING SEQUENCE:
;    precess_array, ra, dec, equinox1, equinox2, PRINT = print
; INPUT:
;    ra,dec    = ra,dec array
; OUTPUT:
;    ra,dec    = precessed ra,dec array
; OPTIONAL INPUT:
;    None
; INPUT KEYWORDS:
;    equinox1      = starting equinox
;    equinox2      = ending equinox
; OUTPUT KEYWORDS:
;    None
; SIDE EFFECTS:
;    None
; EXAMPLES:
;    precess_array, ra, dec, equinox1, equinox2
; RESTRICTIONS:
;    None
; PROCEDURES CALLED:
;    precess
; REVISION HISTORY:
;    Created by Jean-Philippe Bernard a long time ago, to fix limitation of precess
;-

IF n_params() EQ 0 THEN BEGIN
  print,'precess_array, ra, dec, equinox1, equinox2 [,/print]'
  goto,finished
ENDIF

sira=size(ra) & Ncoo=sira(1)
si=size(sira) & Nsi=si(1)

IF isarray(ra) THEN BEGIN
  Ncoo=sira(Nsi-1)
  nra=reform(ra,Ncoo)
  ndec=reform(dec,Ncoo)

  IF NOT isarray(equinox1) THEN BEGIN
    equi1=replicate(equinox1,Ncoo)
  ENDIF ELSE BEGIN
    equi1=equinox1
  ENDELSE
  IF NOT isarray(equinox2) THEN BEGIN
    equi2=replicate(equinox2,Ncoo)  
  ENDIF ELSE BEGIN
    equi2=equinox2
  ENDELSE

  FOR i=0l,Ncoo-1 DO BEGIN
    rra=nra(i) & ddec=ndec(i)
    precess,rra,ddec,equi1(i), equi2(i), PRINT = print
    nra(i)=rra & ndec(i)=ddec
  ENDFOR

  ra=reform(nra,sira(1:sira(0)))
  dec=reform(ndec,sira(1:sira(0)))
ENDIF ELSE BEGIN
  precess, ra, dec, equinox1, equinox2, PRINT = print
ENDELSE

finished:

END
