FUNCTION minimize_chi2,function_name,param_start,param_range,prec_req, $
                       retro=retro,minchi2=minchi2,itermax=itermax, $
                       tobefit=tobefit, all_chi2_ptr=all_chi2_ptr, $
                       N_range=N_range,param_names=param_names,format=format,do_print=do_print
;+
; NAME:
;      minimize_chi2
; CALLING SEQUENCE:
;      minimize_chi2,function_name,param_start,param_range,prec_req, $
;                       retro=retro,minchi2=minchi2,itermax=itermax
; PURPOSE:
;       minimize chi2 using zooming method in parameter space
; INPUTS:
; OPTIONAL INPUT:
;	   None
; OUTPUTS:
; PROCEDURE AND SUBROUTINE USED
; SIDE EFFECTS:
; MODIFICATION HISTORY:
;       written by Jean-Philippe Bernard 09-92
;       Modified by JPB on Dec 19th 03 to set some of the parameters
;       (tobefit)
;       Modified by JPB on Dec 19th 03 to output all chi2 values
;       Modified by JPB on Dec 22nd 03 to print parameter values
; EXAMPLE:
;-

NNparams=n_elements(param_start)

sstr='-> '
IF keyword_set(param_names) THEN BEGIN
  FOR i=0L,NNparams-1 DO BEGIN
    IF i NE 0 THEN BEGIN
      sstr=sstr+','+param_names(i)
    ENDIF ELSE BEGIN
      sstr=sstr+param_names(i)
    ENDELSE
  ENDFOR
  sstr=sstr+' ='
ENDIF

sstrlen=strlen(sstr)

IF keyword_set(format) THEN BEGIN
  frmt=format
ENDIF ELSE BEGIN
  frmt="'(A"+strtrim(sstrlen,2)+','
  FOR i=0L,NNparams DO BEGIN
    frmt=frmt+'E9.2'
    IF i NE NNparams-1 THEN BEGIN
      frmt=frmt+','
    ENDIF ELSE BEGIN
      frmt=frmt+')'
    ENDELSE
  ENDFOR
ENDELSE

IF NOT keyword_set(N_range) THEN BEGIN
  Nv = 5L                      ;number of points used on each axis
ENDIF ELSE BEGIN
  Nv=N_range
ENDELSE

IF not keyword_set(tobefit) THEN BEGIN
  new_start=param_start
  new_range=param_range
  Nparams    = n_elements(new_start)
  ind_2bfit=lindgen(Nparams)
  ind_n2bfit=-1 & count_n2bfit=0
ENDIF ELSE BEGIN
  ind_2bfit=where(tobefit EQ 1,count_2bfit)
  ind_n2bfit=where(tobefit NE 1,count_n2bfit)
  new_start=param_start(ind_2bfit)
  new_range=param_range(ind_2bfit)
  Nparams    = n_elements(new_start)
ENDELSE

all_params = fltarr(NNparams,Nv)
Ncombi     = Nv^Nparams
all_chi2   = fltarr(Ncombi)
;cur_param  = fltarr(NNparams)
cur_param  = param_start
prec       = fltarr(Nparams)
count_prec = 1
print,strtrim([new_start,new_range,prec],2)
iter = 0
IF NOT keyword_set(itermax) THEN itermax = 1000
all_chi2_ptr=ptrarr(itermax*Ncombi,2)

;WHILE count_prec NE 0 AND iter LT itermax AND $
;      abs(max(new_range/param_range)) LE 2 DO BEGIN
WHILE count_prec NE 0 AND iter LT itermax OR $
      iter EQ 0 DO BEGIN
  iter = iter+1
  FOR i=0,Nparams-1 DO BEGIN
    par_min = new_start(i) - new_range(i)/2.
    par_max = new_start(i) + new_range(i)/2.
    all_params(ind_2bfit(i),*) = findgen(Nv) / (1.*(Nv-1)) * (par_max-par_min) + par_min
  ENDFOR
  IF count_n2bfit NE 0 THEN BEGIN
    FOR ii=0L,Nv-1 DO BEGIN
      all_params(ind_n2bfit,ii) = param_start(ind_n2bfit)
    ENDFOR
  ENDIF
  FOR k=0,Ncombi-1 DO BEGIN
    IF Nparams EQ 1 THEN BEGIN
      FOR p=0,NNparams-1 DO BEGIN
        cur_param(p) = all_params(p,k)
      ENDFOR
    ENDIF ELSE BEGIN
      ij = index2ij([k],replicate(Nv,Nparams))
;      stop
      FOR p=0,Nparams-1 DO BEGIN
        cur_param(ind_2bfit(p)) = all_params(ind_2bfit(p),ij(0,p))
      ENDFOR
    ENDELSE
    str         = 'chi2='+function_name+'(cur_param)'
    toto        = execute(str)
;    stop
    all_chi2(k) = chi2
    all_chi2_ptr((iter-1)*Ncombi+k,0)=ptr_new(chi2)
    all_chi2_ptr((iter-1)*Ncombi+k,1)=ptr_new(cur_param)
  ENDFOR
  chi2 = reform(all_chi2,replicate(Nv,Nparams))
  minchi2 = min(chi2)
  ind  = where(chi2 EQ minchi2,count)
  IF count EQ 0 THEN stop
  ij   = index2ij(ind,(size(chi2))(1:Nparams))
  ij   = ij(0,*)
;  print,ij

;
;== check if minimum chi2 is at edge of cube
  IF keyword_set(retro) THEN BEGIN
    ind_edge = where(ij EQ 0 OR ij EQ Nv-1,count_edge)
  ENDIF ELSE BEGIN
    count_edge = 0
  ENDELSE
;  stop
  parms=all_params(*,0)
  FOR i=0,Nparams-1 DO BEGIN
    parms(ind_2bfit(i)) = all_params(ind_2bfit(i),ij(i))
  ENDFOR
  FOR i=0,Nparams-1 DO BEGIN
    new_start(i) = all_params(ind_2bfit(i),ij(i))
  ENDFOR
  IF count_edge EQ 0 THEN BEGIN
 ;== set new parameters values and decrease range
    FOR i=0,Nparams-1 DO BEGIN
      new_range(i) = new_range(i)/2.
    ENDFOR
  ENDIF ELSE BEGIN
;== set new parameters values and
;increase range of bumping parameters
    new_range(ind_edge) = new_range(ind_edge)*2.
  ENDELSE
;== modif
;sauvage...-----------------------------------------------------------------
;  new_s = new_start
;  ind_ns = where(abs(new_s) LT .5,count_ns)
;  IF count_ns GT 0 THEN new_s(ind_ns) = 1.
;  prec    = abs(new_range/new_s)*100. ;in %
;-------------------------------------------------------------------------------------
  prec    = abs(new_range/new_start)*100. ;in %
  indprec = where(prec GT prec_req,count_prec)
  ;print,strtrim([new_start,new_range,prec],2)
  IF keyword_set(do_print) THEN BEGIN
    exstr='str=string(sstr,'
    FOR i=0L,NNparams-1 DO BEGIN
      exstr=exstr+'parms('+strtrim(i,2)+')'
      IF i NE NNparams-1 THEN BEGIN
        exstr=exstr+','
      ENDIF ELSE BEGIN
        exstr=exstr+',format=frmt)'
      ENDELSE
    ENDFOR
;    stop
    toto=execute(exstr)
;    message,str,/info
    print,str
  ENDIF
ENDWHILE
;print,strtrim([new_start,new_range,prec],2)
IF count_prec NE 0 THEN print,'Minimization aborted : too many iterations !'
;RETURN,new_start
result=param_start
result(ind_2bfit)=new_start

all_chi2_ptr=all_chi2_ptr(0:iter*Ncombi-1,*)

RETURN,result
END
