PRO write_xcat,struct,filename,silent=silent

;+
; NAME:
;       write_xcat
; CALLING SEQUENCE:
;       write_xcat,struct,filename
; PURPOSE:
;       write an IDL structure into an ASCII IPAC format file
; INPUTS:
;       filename = file name
;       struct   = IDL structure
; OPTIONAL INPUT:
;       None
; OUTPUTS:
;	   None
; PROCEDURE AND SUBROUTINE USED
;
; SIDE EFFECTS:
;       The file is created
; MODIFICATION HISTORY:
;       written by Jean-Philippe Bernard 16-May-97 @ IAS
; EXAMPLE:
;       file=!jpblib_example_dir+'xcat_example.xcat'
;       st=read_xcat(file)
;       help,st,/str
;       file_out=!jpblib_example_dir+'xcat_output.xcat'
;       write_xcat,st,file_out
;-

IF N_PARAMS(0) LT 1 OR N_PARAMS(0) GT 2 THEN BEGIN
  print,'Calling Sequence: write_xcat,struct,filename'
  print,'Accepted K-words: '
  GOTO,sortie
ENDIF

tags=tag_names(struct)

;=== write the xcat header
si=size(struct) & Nline=si(1)
;stop
n=n_struct(struct,ntags)
tyyp=intarr(Ntags)
FOR i=0,Ntags-1 DO BEGIN
  si=size([struct.(i)])
  tyyp(i)=si(2)
ENDFOR
tags_typ=strarr(Ntags)
ind=where(tyyp EQ 7,count)
IF count NE 0 THEN tags_typ(ind)='char'
ind=where(tyyp EQ 2,count)
IF count NE 0 THEN tags_typ(ind)='int'
ind=where(tyyp EQ 3,count)
IF count NE 0 THEN tags_typ(ind)='long'
ind=where(tyyp EQ 4,count)
IF count NE 0 THEN tags_typ(ind)='real'
ind=where(tyyp EQ 5,count)
IF count NE 0 THEN tags_typ(ind)='double'

;Find out length of each column
len=intarr(Ntags)
FOR i=0,Ntags-1 DO BEGIN
  str=strtrim(struct.(i),2)
  strr=strtrim(tags_typ(i))
  strrr=strtrim(tags(i))
  len(i)=max([strlen(str),strlen(strr),strlen(strrr)])+1
ENDFOR
str1='' & str2='|' & str3='|' & str5='|'
FOR i=0,total(len)+Ntags-1 DO str1=str1+'='
FOR i=0,Ntags-1 DO BEGIN
  str2=str2+tags_typ(i)
  FOR j=strlen(tags_typ(i)),len(i)-1 DO BEGIN
    str2=str2+' '
  ENDFOR
  str2=str2+'|'
  str3=str3+tags(i)
  FOR j=strlen(tags(i)),len(i)-1 DO BEGIN
    str3=str3+' '
  ENDFOR
  str3=str3+'|'
  str5=str5+'NULL'
  FOR j=strlen('NULL'),len(i)-1 DO BEGIN
    str5=str5+' '
  ENDFOR
  str5=str5+'|'
ENDFOR
str4=str1
IF not keyword_set(silent) THEN BEGIN
  print,str1
  print,str3
  print,str2
  print,str5
  print,str4
ENDIF
FOR i=0,Nline-1 DO BEGIN
  str=''
  FOR j=0,Ntags-1 DO BEGIN
    strr=strtrim(struct(i).(j),1)
    str=str+strr
    FOR k=strlen(strr),len(j)-1 DO BEGIN
      str=str+' '
    ENDFOR
    str=str+' '
  ENDFOR
  IF not keyword_set(silent) THEN print,str
ENDFOR

;now write the file
comment='\Written by write_xcat.pro on '+systime(0)
GET_LUN,unit
openw,unit,filename
printf,unit,comment
printf,unit,str3
printf,unit,str2
printf,unit,str5
FOR i=0,Nline-1 DO BEGIN
  str=''
  FOR j=0,Ntags-1 DO BEGIN
    strr=strtrim(struct(i).(j),1)
    str=str+strr
    FOR k=strlen(strr),len(j)-1 DO BEGIN
      str=str+' '
    ENDFOR
    str=str+' '
  ENDFOR
  printf,unit,str
ENDFOR

close,unit
free_lun,unit

sortie:

END
