PRO xhi2_error,xhi2,values,errors,Ndata,conf_lev=conf_lev,itmax=itmax,dxhi=dxhi

;+
; NAME:
;	xhi2_error
; PURPOSE:
;    
; CATEGORY:
;    JPBLib
; CALLING SEQUENCE: 
;	xhi2_error,xhi2,values,errors,Ndata,conf_lev=conf_lev,itmax=itmax,dxhi=dxhi
; INPUTS:
;    xhi2 = array of xhi2 values
;    Ndata = Number of independant data points
; OPTIONAL INPUT PARAMETERS:
;    conf_lev = confidence level requiered (default 68.3 %)
;    it_max   = 
; OUTPUTS:
;    values  = values corresponding to min of xhi2
;    errors  = error on the values
; OPTIONAL OUTPUT PARAMETERS:
;    dxhi    = xhi increase corresponding to the requested confidence level
; ACCEPTED KEY-WORDS:
; COMMON BLOCKS:
; SIDE EFFECTS:
; RESTRICTIONS:
; PROCEDURE:
; MODIFICATION HISTORY:
;	Written Jan 31 1995 J.P. Bernard, IPAC.
;-

IF N_PARAMS(0) LT 3 THEN BEGIN
  PRINT,'Calling Sequence: '
  PRINT,'xhi2_error,xhi2,values,errors,Ndata[,conf_lev=][,itmax=][,dxhi=]'
  GOTO, closing
ENDIF

IF NOT keyword_set(conf_lev) THEN BEGIN
  conf_level=68.3
ENDIF ELSE BEGIN
  conf_level=conf_lev
ENDELSE

si=size(xhi2)
Ndim=si(0)
values=lonarr(Ndim) & errors=lonarr(Ndim,2)
siz=si(1:Ndim)

ximin=min(xhi2) & ximax=max(xhi2)

ind=where(xhi2 EQ ximin)
ij=index2ij(ind,siz)
values=ij(0,*)

;Compute range of xhi2 corresponding to the given confidence level

ndl=Ndata-Ndim        ;Number of freedom degrees

Npp=1000
dxhiarray=findgen(Npp)/(1.*Npp)*(ximax-ximin)
pp=fltarr(Npp)
FOR i=0,Npp-1 DO pp(i)= $
        (1.-nr_gammaq(ndl/2.,dxhiarray(i)/2.,itmax=itmax))*100.  ;in %
order=sort(pp) & pp=pp(order) & dxhiarray=dxhiarray(order)
dxhi=interpol(dxhiarray,pp,conf_level)
plot,pp,dxhiarray,xtit='P(xhi2>min(xhi2)+dxhi2) (%)',ytit='dxhi2'
oplot,[conf_level,conf_level],minmax(dxhiarray),psym=-3
;stop

xhi=ximin+dxhi

ind=where(xhi2 LE xhi(0),count)

;=== For each dimension, project ind
ijs=index2ij(ind,siz)
FOR i=0,Ndim-1 DO BEGIN
  errors(i,0)=min(ijs(*,i))
  errors(i,1)=max(ijs(*,i))
ENDFOR

closing:

END




