PRO make_issa_cat,cd=cd

;+
; NAME:
;      make_issa_cat
; PURPOSE:
;      Generates a catalog of positions for all ISSA plates
;      to be used by locate_issa.pro
; CATEGORY:
;      ISSA
; CALLING SEQUENCE:
;      make_issa_cat[,/cd]
; INPUTS:
;      None
; OPTIONAL INPUTS:
;      cd = name for a CD where to look for ISSA files
; KEYWORD PARAMETERS:
;      None
; OUTPUTS:
;      None
; OPTIONAL OUTPUTS:
;      None
; COMMON BLOCKS:
;      None
; SIDE EFFECTS:
;      Generates an Xcatscan-like file in !jpblib_issa_dir
; RESTRICTIONS:
;      Must have write permission
; PROCEDURE:
;      Uses write_xcat.pro
; EXAMPLE:
;      make_issa_cat,cd='Xcaret6G:Data:IRAS'
; MODIFICATION HISTORY:
;      Created Oct 30 1997 by J.Ph. Bernard IAS
;      June 2 1998, Modified for MacOS by JPB
;      July 17 1998, Modified JPB to include cd keyword
;-

IF NOT keyword_set(cd) THEN cdname='ISSA_0001' ELSE cdname=cd
IF cdname EQ 'ISSA_0001' THEN cdnum='1' ELSE cdnum='0'

CASE !version.os OF
  'OSF':BEGIN
;     ddir='/net/castor/issa/'
     ddir='/net/orion/data/issa/'
     paths=['north/IMAGES/','north/REJECT/','south/IMAGES/','south/REJECT/']
     ichar='I' & bchar='B' & extchar='.FIT*'
  END
  'hp-ux':BEGIN
;     ddir='/net/castor/issa/'
     ddir='/net/orion/data/issa/'
     paths=['north/IMAGES/','north/REJECT/','south/IMAGES/','south/REJECT/']
     ichar='I' & bchar='B' & extchar='.FIT*'
  END
  'MacOS':BEGIN  ;Assumes ISSA CDROM#1 is mounted
    ddir=!iras_data_dir
    paths=['']
    ichar='I' & bchar='B' & extchar='.FIT'
  END
  'darwin':BEGIN
    ddir=!iras_data_dir
    paths=['']
    ichar='I' & bchar='B' & extchar='.FIT'
  END
ENDCASE

Npaths=(size(paths))(1)

;============= This is for North images
st0={filenames:' ',crval1:0.,crval2:0.}
tags=['filenames','crval1','crval2']
all_names=[' '] & all_crval1=[!indef] & all_crval2=[!indef]

FOR k=0,Npaths-1 DO BEGIN
  dir1=ddir+paths(k)
  list=findfile(dir1+ichar+'*'+bchar+'1*'+extchar,count=count)
;  st=replicate(st0,count)

  FOR i=0,count-1 DO BEGIN
;  FOR i=0,0 DO BEGIN
    print,list(i)
;    IF !version.os EQ 'hp-ux' THEN d=readfits(list(i),h,/honly) ELSE $
    d=readfits(list(i),h)
    pos=strposmulti(list(i),!sep,ccount)
    toto=strmid(list(i),pos(ccount-1)+1,strlen(list(i)))
;   toto=list(i)
    all_names=[all_names,toto]    
    all_crval1=[all_crval1,sxpar(h,'CRVAL1')]
    all_crval2=[all_crval2,sxpar(h,'CRVAL2')]
  ENDFOR
END
all_names=all_names(1:*) & all_crval1=all_crval1(1:*) & all_crval2=all_crval2(1:*)
si=size(all_crval1) & Nel=si(1)

st=replicate(st0,Nel)
st.filenames=all_names
st.crval1=all_crval1
st.crval2=all_crval2


cat_name='issa_all_pos.xcat'
;IF !version.os EQ 'MacOS' THEN cat_name='issa_all_pos'+cdnum+'.xcat'

write_xcat,st,!jpblib_issa_dir+cat_name

END
