PRO pistou,vec,exth,nside=nside,lon_center=lon_center,lat_center=lat_center, $
           min=min,max=max,log=log,partial=partial,xyz=xyz,grid_color=grid_color, $
           color_table=color_table

;+
; NAME:
;       pistou
; CALLING SEQUENCE:
;       pistou,vec,exth,nside=nside,lon_center=lon_center,lat_center=lat_center, $
;       min=min,max=max,log=log,partial=partial,xyz=xyz,grid_color=grid_color, $
;       color_table=color_table
; PURPOSE:
;       Simple interface between IDL and Pistou V0.9
; INPUTS:
;       vec	= Healpix vector to be represented
;       exth	= Healpix header
; OPTIONAL INPUT:
;	None
; ACCEPTED KEY-WORDS:
;	lon_center  = longitude on which the view should be centered(degree)
;	lat_center  = latitude on which the view should be centered(degree)
;       min         = minimum value to be displayed
;       max         = maximum value to be displayed
;	log         = if set, a log color table is used
;       partial     = if set !indef values are not used and partial healpix is used.
;       xyz         = [x,y,z] normalized vector for orientation
;       grid_color  = color of the grid (limited to ['red','green','blue'] for now)
;       color_table = idl color table used for the map
; EXAMPLES
; OUTPUTS:
;       None
; PROCEDURE AND SUBROUTINE USED
;	IDL PROCEDURE used:
; COMMONS:
; SIDE EFFECTS:
; MODIFICATION HISTORY:
;    written March 13th 2005 by Jean-Philippe Bernard, CESR.
;-

;---------------------------------------------------------------
; parameter check
;---------------------------------------------------------------
IF N_PARAMS(0) LT 1 THEN BEGIN
  PRINT,'Calling Sequence: '
  PRINT,'pistou,vec,exth,[,nside=][,lon_center=][,lat_center=][,min=][,max=][,/log][,/partial][,xyz=[]]'
  PRINT,'                [,grid_color=][,color_table=]'
  GOTO, sortie
ENDIF

gen_vec_id='"1"'
gen_vec_show="true"
gen_vec_lock="false"
gen_vec_position=0
gen_vec_fieldInd='"-1"'
gen_vec_partial='"0"'
gen_vec_pixelisation="HealPix"
gen_vec_file="/Data/Pistou_Examples/Pistou_sample_data/TQU.fits"
gen_vec_coordsys="ECL"
gen_vec_color="#ff0000"
gen_vec_powNsideInitDisplay=3
gen_vec_trans_min=0
gen_vec_trans_max=1

IF keyword_set(xyz) THEN BEGIN
  x_up=xyz(0)  & y_up=xyz(1) & z_up=xyz(2)
ENDIF ELSE BEGIN
  x_up=0. & y_up=0. & z_up=1.
ENDELSE
IF keyword_set(grid_color) THEN BEGIN
  CASE strlowcase(grid_color) OF
    'red': BEGIN
       use_grid_color='#ff0000'
    END
    'green':BEGIN
       use_grid_color='#00ff00'
    END
    'blue':BEGIN
       use_grid_color='#0000ff'
    END
    ELSE: BEGIN
      message,'Unrecognized grid color table'
    END
  ENDCASE
ENDIF ELSE BEGIN
  use_grid_color='#ff0000'    ;red is the default
ENDELSE

color_table_dir=getenv('HOME')+'/.pistou/ppm/'
IF keyword_set(color_table) THEN BEGIN
  CASE color_table OF
    0: BEGIN
       use_color_table='B-W_LINEAR'
    END
    1:BEGIN
       use_color_table='BW'
    END
    2:BEGIN
       use_color_table='BW'
    END
    3:BEGIN
       use_color_table='RED_TEMPERATURE'
    END
    ELSE:BEGIN
      message,'Unrecognized color_table'
    END
  ENDCASE
ENDIF ELSE BEGIN
  use_color_table='B-W_LINEAR'    ;is the default
ENDELSE
use_color_table=color_table_dir+use_color_table+'.ppm'

CASE (size(vec))(0) OF
  1:BEGIN
    polar=0
  END
  3:BEGIN
    polar=1
  END
  ELSE:BEGIN
    message,'Invalid input array dimensions',/info
    goto,sortie
  END
ENDCASE

Npix=(size(vec))(1)
nside=npix2nside(Npix)
IF nside EQ -1 THEN BEGIN
  message,'Unrecognized number of pixels',/warning
  goto,sortie
ENDIF

;IF keyword_set(exth) THEN BEGIN
  header=exth
  use_nside=sxpar(header,'NSIDE',count=count_nside)
  IF count_nside EQ 0 THEN BEGIN
    message,'Could not find NSIDE in provided header',/info
    goto,sortie
  ENDIF
  ordering=sxpar(header,'ORDERING',count=count_ordering)
  IF count_ordering EQ 0 THEN BEGIN
    message,'Could not find ORDERING in provided header',/info
    goto,sortie
  ENDIF
  coordsys=sxpar(header,'COORDSYS',count=count_coordsys)
  IF count_coordsys EQ 0 THEN BEGIN
    message,'Could not find COORDSYS in provided header',/info
    goto,sortie
  ENDIF
;ENDIF ELSE BEGIN
;  IF not keyword_set(nside) THEN BEGIN
;    npix=n_elements(vec)
;    use_Nside=npix2nside(npix)
;  ENDIF ELSE BEGIN
;    use_Nside=nside
;  ENDELSE
;  coordsys='E'
;  ordering='NESTED'
;  header=make_healp_header(use_nside,coordsys=coordsys,ordering=ordering, $
;                           main_header=main_header,data_sample=data_sample)
;ENDELSE

;store the input variable into an appropriate file
fits_file='/tmp/pistou_file.fits'
IF polar EQ 0 THEN BEGIN
  IF not keyword_set(partial) THEN BEGIN
    write_fits_map,fits_file,vec,header
  ENDIF ELSE BEGIN
;    stop
    ind=where(vec NE !indef,count)
    all_pixels=lindgen(nside2npix(use_nside))
    pixels=all_pixels(ind)
    data=vec(ind)
    WRITE_FITS_CUT4,fits_file,pixels,data,nside=use_nside,ordering=sxpar(header,'ORDERING'),coordsys=sxpar(header,'COORDSYS')
  ENDELSE
ENDIF ELSE BEGIN
  one_st={Temperature:0.,Q_Polarisation:0.,Q_Polarisation:0.}
  st=replicate(one_st,Npix)
  st.(0)=vec(*,0)
  st.(1)=vec(*,1)
  st.(2)=vec(*,2)
  mwrfits,st,filts_file,h,/create
  sxaddpar,h,'NSIDE',nside,before='TFORM1'
  sxaddpar,h,'COORDSYS',coordsys,before='TFORM1'
  sxaddpar,h,'ORDERING',ordering,before='TFORM1'
  sxaddpar,h,'PIXTYPE','HEALPIX',before='TFORM1'
  sxaddpar,h,'POLAR','T',before='TFORM1'
  mwrfits,st,filts_file,h,/create
ENDELSE

;=== retrieve a example session .xml file (single Intensity sphere)
file=getenv('JPBLIB_DIR')+'/Image_Display/Pistou/'+'gen_pistou_session1.xml'
strs_in=file2strings(file)
strs_in=modify_pistou_xml(strs_in,file=fits_file,lon_center=lon_center,lat_center=lat_center, $
                     min=min,max=max,log=log,x_up=x_up,y_up=y_up,z_up=z_up,grid_color=use_grid_color, $
                     color_table=use_color_table)
strs=strs_in
IF polar EQ 1 THEN BEGIN
  file=getenv('JPBLIB_DIR')+'/Image_Display/Pistou/'+'gen_pistou_polar1.xml'
  strs_pol=file2strings(file)
  strs_pol=modify_pistou_xml(strs_pol,file=fits_file)
  ;will have to merge that in the string ....
ENDIF

;stop

;=== define the output .xml file
xml_out='/tmp/pistou_session.xml'
strings2file,strs,xml_out

;spawn,'pistou '+file+' &'
spawn,'pistou '+xml_out+' &'

sortie:

END
