FUNCTION create_contour_obj_st,images,silent=silent,_extra=extra

;+
; NAME:
;         create_contour_obj_st
; PURPOSE:
;         Creates a default structure for contour plotting with JPBlibimcont objects
; CATEGORY:
;         image display
; CALLING SEQUENCE:
;         st=create_contour_obj_st(images,[/silent][,lev1=lev1][,lev2=lev2],...)
; INPUTS:
;         images  : images to be contoured. Can be a cube of several images
; OPTIONAL INPUTS:
;         lev1    : first set of level values (for 1st image)
;         lev2    : 2nd level of values (for 2nd image), ...
; KEYWORD PARAMETERS:
;         silent  : if set the routine is silent
; OUTPUTS:
;         st      : Overlay structure
; OPTIONAL OUTPUTS:
;         None
; COMMON BLOCKS:
;         None
; SIDE EFFECTS:
;         None
; RESTRICTIONS:
;         None
; PROCEDURE:
;         Straigthforward.
;         Uses create_struct
; EXAMPLE:
;         images=fltarr(10,13,3)
;         lev1=[1,2,3,4] & lev2=[0.1,0.6,0.7]
;         levels=create_contour_obj_st(images,lev1=lev1,lev2=lev2)
;         help,levels,/str
;         print,levels.set0.values
; MODIFICATION HISTORY:
;         Created by J.Ph Bernard @ IAS on Jul 30 1997
;-

;On line help
IF n_params(0) LT 1 THEN BEGIN
    print,'Use: levels=create_contour_obj_st(image)'
    print,'Kwords: silent,lev1=[],lev2=[],...,lev5=[]'
    GOTO,sortie
ENDIF

Nmax_lev=30 ;default number of levels

si=size(images)
N_set=1 & xstr=strtrim(si(1),2) & ystr=strtrim(si(2),2)
IF si(0) GE 3 THEN BEGIN
  N_set=si(3)
ENDIF
tagnames=tag_names(extra)
ind=where(strlowcase(strmid(tagnames,0,3)) EQ 'lev',Nlevset)
IF Nlevset NE N_set THEN BEGIN
  message,'Incorrect number of valid contour set found',/info
  goto,sortie
ENDIF

N_values=lindgen(N_set)
FOR i=0,N_set-1 DO BEGIN
  N_values(i)=(size(extra.(ind(i))))(1)
ENDFOR

Nchar=8
;seed=0
sttr='st={'
tagnames=['image','value','color','thick','linestyle', $
           'label','double','spline', $
           'downhill','missing','charsize']
FOR i=0,N_set-1 DO BEGIN
    nstr=strtrim(string(n_values(i)),2)
    istr=strtrim(string(i),2)
    tag_typ='F('+xstr+','+ystr+ $
      '),F('+nstr+'),A('+nstr+'),I('+nstr+'),I('+nstr+'),A('+ $
      nstr+'),I('+nstr+'),I('+nstr+'),I('+nstr+'),F('+nstr+'),I('+nstr+')'
    name=''
    str='my_create_struct,st'+istr+',name,tagnames,tag_typ'
    truc=execute(str)
    sttr=sttr+'set'+istr+':st'+istr
    IF i NE N_set-1 THEN sttr=sttr+','
ENDFOR
sttr=sttr+'}'

truc=execute(sttr)

;clean up the mess
;IF !version.os EQ 'MacOS' THEN remove_tmp_files

;Fill structure with default values
FOR i=0,N_set-1 DO BEGIN
    st.(i).value=extra.(i)
ENDFOR

FOR i=0,N_set-1 DO BEGIN
  st.(i).image=images(*,*,i)
  st.(i).missing=!indef
  st.(i).thick=1
  st.(i).color='black'       ;0
  st.(i).linestyle=0         ;0
  st.(i).spline=1            ;1
  st.(i).charsize=1          ;1
  st.(i).downhill=0
  st.(i).double=0
  st.(i).label=''
ENDFOR

RETURN,st
sortie:
END
