FUNCTION create_contour_st,images,silent=silent,lev1=lev1,lev2=lev2,lev3=lev3, $
            lev4=lev4,lev5=lev5,lev6=lev6

;+
; NAME:
;         create_contour_st
; PURPOSE:
;         Creates a default structure for contour plotting in image_cont20
; CATEGORY:
;         image display
; CALLING SEQUENCE:
;         st=create_contour_st(images,[/silent][,lev1=lev1][,lev2=lev2],...)
; INPUTS:
;         images  : images to be contoured. Can be a cube of several images
; OPTIONAL INPUTS:
;         lev1    : first set of level values (for 1st image)
;         lev2    : 2nd level of values (for 2nd image)
; KEYWORD PARAMETERS:
;         silent  : if set the routine is silent
; OUTPUTS:
;         st      : Overlay structure
; OPTIONAL OUTPUTS:
;         None
; COMMON BLOCKS:
;         None
; SIDE EFFECTS:
;         None
; RESTRICTIONS:
;         None
; PROCEDURE:
;         Straigthforward.
;         Uses create_struct
; EXAMPLE:
;         images=fltarr(10,13,3)
;         lev1=[1,2,3,4] & lev2=[0.1,0.6,0.7]
;         levels=create_contour_st(images,lev1=lev1,lev2=lev2)
;         help,levels,/str
;         print,levels.set0.values
; MODIFICATION HISTORY:
;         Created by J.Ph Bernard @ IAS on Jul 30 1997
;         Modified JPB 19/02/00 to remove call to remove_tmp_files
;-

;On line help
IF n_params(0) LT 1 THEN BEGIN
    print,'Use: levels=create_contour_st(image)'
    print,'Kwords: silent,lev1=[],lev2=[],...,lev5=[]'
    GOTO,sortie
ENDIF

Nmax_lev=30 ;default number of levels

si=size(images)
N_set=1 & xstr=strtrim(si(1),2) & ystr=strtrim(si(2),2)
IF si(0) GE 3 THEN BEGIN
  N_set=si(3)
ENDIF

N_values=fix([!indef])
FOR i=0,N_set-1 DO BEGIN
  str='toto=keyword_set(lev'+strtrim(i+1,2)+')'
  truc=execute(str)
  IF toto THEN BEGIN
    str='si=size(lev'+strtrim(i+1,2)+')'
    truc=execute(str)
    N_values=[N_values,si(1)]
  ENDIF ELSE BEGIN
    N_values=[N_values,Nmax_lev]
  ENDELSE
ENDFOR
N_values=N_values(1:*)

;stop
;si=size(N_values) & N_set=si(1)

Nchar=8
;seed=0
sttr='st={'
tagnames=['image','value','color','thick','linestyle', $
           'label','double','spline', $
           'downhill','missing','charsize']
FOR i=0,N_set-1 DO BEGIN
    nstr=strtrim(string(n_values(i)),2)
    istr=strtrim(string(i),2)
    tag_typ='F('+xstr+','+ystr+ $
      '),F('+nstr+'),I('+nstr+'),I('+nstr+'),I('+nstr+'),A('+ $
      nstr+'),I('+nstr+'),I('+nstr+'),I('+nstr+'),F('+nstr+'),I('+nstr+')'
;    IF not keyword_set(silent) THEN  print,tag_typ
;    name='st'      
;    FOR j=0,Nchar DO BEGIN
;        name=name+strtrim(string(fix(randomu(seed)*10)),2)
;    ENDFOR
;  name='st'+istr
;   stop
    name=''
    str='my_create_struct,st'+istr+',name,tagnames,tag_typ'
;  stop
    truc=execute(str)
    sttr=sttr+'set'+istr+':st'+istr
    IF i NE N_set-1 THEN sttr=sttr+','
ENDFOR
sttr=sttr+'}'

;stop
truc=execute(sttr)

;clean up the mess
;IF !version.os EQ 'MacOS' THEN remove_tmp_files

;Fill structure with default values
FOR i=0,N_set-1 DO BEGIN
  str='toto=keyword_set(lev'+strtrim(i+1,2)+')'
  truc=execute(str)
  IF toto THEN BEGIN
    str='levels=lev'+strtrim(i+1,2)
    truc=execute(str)
    st.(i).value=levels
  ENDIF ELSE BEGIN
    st.(i).value(*)=!indef
  ENDELSE
ENDFOR

FOR i=0,N_set-1 DO BEGIN
  st.(i).image=images(*,*,i)
  st.(i).missing=!indef
  st.(i).thick=1
  st.(i).color=0           ;0
  st.(i).linestyle=0         ;0
  st.(i).spline=1            ;1
  st.(i).charsize=1          ;1
  st.(i).downhill=0
  st.(i).double=0
  st.(i).label=''
ENDFOR

RETURN,st
sortie:
END
