PRO disp_spectra,input,header,imin,imax,jmin,jmax,HISTO=histo,VRANGE=vrange $
                ,TRANGE=trange,VMARK=vmark,TMARK=tmark,REBIN1=rebin1, $
                 COORD=coord $
                ,thick1=thick1,style=style $
		,input2=input2,header2=header2,scale2=scale2,offset2=offset2 $
		,color2=color2,thick2=thick2,style2=style2,rebin2=rebin2 $
                ,input3=input3,header3=header3,scale3=scale3,offset3=offset3 $
                ,color3=color3,thick3=thick3,style3=style3,rebin3=rebin3 $
                ,csize=csize

;+
; NAME:
;       disp_spectra
; CALLING SEQUENCE:
;       disp_spectra,input,header,imin,imax,jmin,jmax
; PURPOSE:
;       display a set of spectra 
; INPUTS:
;       input = cube (3D) in which data are taken.
;       header = cube's header
;       imin,imax = interval on first axis
;	jmin,jmax = interval on second axis
; OPTIONAL INPUT:
; ACCEPTED KEY-WORDS:
; 	histo	= if set, plot in histogram mode
;	vrange	= gives the velocity range of plots
;	trange	= gives the Temperature range of plots
;	vmark	= location for vertical bars @ cste velocity
;	tmark	= location for horizontal bars @ cste temp
;	rebin1	= # of channels to rebin
;	coord	= 'ad' or 'lb' or nthg:type of coordinates to print on each plot
;    csize = character size for coordinates
; OUTPUTS:
;       None
; PROCEDURE AND SUBROUTINE USED
;       euler, nice_coord, sxpar,create_lb_fukui, create_coo2
; SIDE EFFECTS:
;       ?
; MODIFICATION HISTORY:
;       written Jean-Philippe Bernard 25-Oct-92
;	modified JPB 06-Nov-1992 to plot real Temperature from cubes
;	modified JPB 17-Feb-1993 to put Keywords HISTO, VRANGE, TRANGE
;	                                        , VMARK, TMARK, REBIN, COORD
;	modified JPB 26-Feb-1993 to remove undef values from min max search
;	modified JPB 16-Mars-1993 to overplot coord at center of vrange.
;	modified JPB Jan-94 to plot 2 arrays (V1.0)
;	modified JPB Fev 2000 to use !indef
;-

;on_error,2

IF N_PARAMS(0) LT 6 THEN begin
  print,'Calling Sequence: disp_spectra,input,header,imin,imax,jmin,jmax'
  print,'Accepted Key-Words:  HISTO, VRANGE, TRANGE, VMARK, TMARK, REBIN1'
  print,'                     COORD[,input2=,header2=][,scale2=,offset2=]'
  print,'                     [color2=]'
  goto,sortie
endif
IF N_elements(input2) NE 0 and n_elements(header2) EQ 0 THEN BEGIN
  print,'header2 requiered'
  goto,sortie
ENDIF
IF N_elements(input3) NE 0 and n_elements(header3) EQ 0 THEN BEGIN
  print,'header3 requiered'
  goto,sortie
ENDIF

ver='1.0'


bunit=strtrim(sxpar(header,'BUNIT'),2)
blank=sxpar(header,'BLANK')
bscale=sxpar(header,'BSCALE')
naxis1=sxpar(header,'NAXIS1')
naxis2=sxpar(header,'NAXIS2')
naxis3=sxpar(header,'NAXIS3')
crval1=sxpar(header,'CRVAL1')
crval2=sxpar(header,'CRVAL2')
crval3=sxpar(header,'CRVAL3')
crpix1=sxpar(header,'CRPIX1')
crpix2=sxpar(header,'CRPIX2')
crpix3=sxpar(header,'CRPIX3')
cdelt1=sxpar(header,'CDELT1')
cdelt2=sxpar(header,'CDELT2')
cdelt3=sxpar(header,'CDELT3')
ctype1=strtrim(sxpar(header,'CTYPE1'),2)
ctype2=strtrim(sxpar(header,'CTYPE2'),2)
ctype3=strtrim(sxpar(header,'CTYPE3'),2)
name=strtrim(sxpar(header,'OBJECT'),2)

pos=strpos(ctype1,'-')
coo_type=strmid(ctype1,0,pos(0))
IF coo_type EQ 'LONG' OR coo_type EQ 'GLON' THEN BEGIN
;  create_lb_fukui,header,l,b,dim=2
  create_coo2,header,l,b,/silent
  euler,l,b,a,d,2
  typ='lb'
ENDIF
print,'ctype1=',ctype1
IF coo_type EQ 'RA' THEN BEGIN
  print,'ad created'
  create_coo2,header,a,d
  euler,a,d,l,b,1
  typ='ad'
ENDIF

gene_marge_x=0.05
gene_marge_y=0.05
taille_x=1.-gene_marge_x
taille_y=1.-gene_marge_y

Nx=imax-imin+1
Tx=taille_x/Nx
Ny=jmax-jmin+1
Ty=taille_y/Ny
!p.multi(0)=Nx*Ny
savchar=!p.charsize
sxsize=!x.charsize
sysize=!y.charsize

NNx=(size(input))(1)
NNy=(size(input))(2)

IF not keyword_set(Trange) THEN BEGIN
;  indef=blank*bscale
;  indef=-32768.
  zone=input(imin:imax,jmin:jmax,*)
  ind=where(zone GT !indef+0.1,count)
  IF count EQ 0 THEN BEGIN
    print,'Array all undefined'
    goto,sortie
  ENDIF
  minval=min(zone(ind))
  maxval=max(zone(ind))
  ;print,'disp_spectra:min=',minval,'max=',maxval
  ;stop
  IF bunit NE 'K' THEN BEGIN
    minval=minval/cdelt3
    maxval=maxval/cdelt3
  ENDIF
ENDIF

erase

!p.charsize=1.
!x.charsize=0.5
!y.charsize=0.5

make_axis,header,ll,bb,vel
minvel=min(vel)
maxvel=max(vel)

ydep=gene_marge_x
xdep=gene_marge_y

;Deal with Key Words
IF (n_elements(histo) EQ 0) THEN BEGIN
  psyym=-3
ENDIF ELSE BEGIN
  psyym=10
ENDELSE
IF (n_elements(rebin1) EQ 0) THEN BEGIN
  rebiin=1
ENDIF ELSE BEGIN
  rebiin=rebin1
ENDELSE
IF (n_elements(vrange) NE 0) THEN BEGIN
  minvel=min(vrange)
  maxvel=max(vrange)
ENDIF
IF (n_elements(trange) NE 0) THEN BEGIN
  minval=min(trange)
  maxval=max(trange)
ENDIF
IF (n_elements(tmark) NE 0) THEN BEGIN
  si=size(tmark)
  nb_tmark=si(1)
  tbary=fltarr(nb_tmark,2)
  tbarx=[minvel,maxvel]
  FOR i=0,nb_tmark-1 DO BEGIN
    tbary(i,*)=tmark(i)
  ENDFOR
ENDIF ELSE BEGIN
  nb_tmark=0
ENDELSE
IF (n_elements(vmark) NE 0) THEN BEGIN
  si=size(vmark)
  nb_vmark=si(1)
  vbarx=fltarr(nb_vmark,2)
  vbary=[minval,maxval]
  FOR i=0,nb_vmark-1 DO BEGIN
    vbarx(i,*)=vmark(i)
  ENDFOR
ENDIF ELSE BEGIN
  nb_vmark=0
ENDELSE
IF N_elements(input2) NE 0 THEN BEGIN
  IF N_elements(scale2) EQ 0 THEN BEGIN
    print,'scale assumed 1'
    sc2=1.
  ENDIF ELSE BEGIN
    sc2=scale2
  ENDELSE
  IF N_elements(offset2) EQ 0 THEN BEGIN
    print,'offset assumed 0'
    of2=0.
  ENDIF ELSE BEGIN
    of2=offset2
  ENDELSE
  IF n_elements(rebin2) NE 0 THEN rebiin2=rebin2 ELSE rebiin2=rebiin
ENDIF
IF N_elements(input3) NE 0 THEN BEGIN
  IF N_elements(scale3) EQ 0 THEN BEGIN
    print,'scale assumed 1'
    sc3=1.
  ENDIF ELSE BEGIN
    sc3=scale3
  ENDELSE
  IF N_elements(offset3) EQ 0 THEN BEGIN
    print,'offset assumed 0'
    of3=0.
  ENDIF ELSE BEGIN
    of3=offset3
  ENDELSE
  IF n_elements(rebin3) NE 0 THEN rebiin3=rebin3 ELSE rebiin3=rebiin
ENDIF
IF not keyword_set(csize) THEN char_size=1 ELSE char_size=csize

;Make the drawing
FOR j=jmin,jmax DO BEGIN
  y1=ydep+Ty*(j-jmin)
  y2=y1+Ty
  FOR i=imin,imax DO BEGIN
    x1=xdep+Tx*(i-imin)
    x2=x1+Tx
    !p.position=[x1,y1,x2,y2]
    !x.tickname=' '
    !y.tickname=' '
    !xtitle=''
    !ytitle=''
    IF j EQ jmin AND i EQ imin THEN BEGIN
      !x.tickname(*)=''
      !y.tickname(*)=''
      !xtitle='v (km/s)'
      !ytitle='T (K)'
    ENDIF
    IF i LE NNx-1 AND j LE NNy-1 THEN BEGIN
      for_plot=reform(input(i,j,*))
    ENDIF ELSE BEGIN
      for_plot=reform(input(0,0,*))
      for_plot(*)=0.
    ENDELSE
    IF bunit NE 'K' THEN BEGIN
      ind=where(for_plot NE !indef,count)
      IF count NE 0 THEN for_plot(ind)=for_plot(ind)/cdelt3
    ENDIF
    plot,vel,for_plot, $
         yrange=[minval,maxval],xrange=[minvel,maxvel], $
         xstyle=1,ystyle=1,/noerase,psym=psyym,nsum=rebiin, $
         thick=thick1,linestyle=style
    IF n_elements(input2) NE 0 THEN BEGIN
      for_plot2=reform(input2(i,j,*))
      IF strtrim(sxpar(header2,'BUNIT'),2) NE 'K' THEN BEGIN
	ind=where(for_plot2 NE !indef,count)
	IF count NE 0 THEN for_plot2(ind)=for_plot2(ind) $
				   /sxpar(header2,'CDELT3')
      ENDIF
      make_axis,header2,ll,bb,v2
      oplot,v2,for_plot2*sc2+of2,color=color2,psym=psyym,nsum=rebiin2, $
               thick=thick2,linestyle=style2
    ENDIF
    IF n_elements(input3) NE 0 THEN BEGIN
      for_plot3=reform(input3(i,j,*))
      IF strtrim(sxpar(header3,'BUNIT'),2) NE 'K' THEN BEGIN
	ind=where(for_plot3 NE !indef,count)
	IF count NE 0 THEN for_plot3(ind)=for_plot3(ind) $
				   /sxpar(header3,'CDELT3')
      ENDIF
      make_axis,header3,ll,bb,v3
      oplot,v3,for_plot3*sc3+of3,color=color3,psym=psyym,nsum=rebiin3, $
               thick=thick3,linestyle=style3
    ENDIF
    IF (n_elements(tmark) NE 0) THEN BEGIN
      FOR k=1,nb_tmark DO BEGIN
        oplot,tbarx,tbary(k-1,*),psym=-3,linestyle=1
      ENDFOR
    ENDIF
    IF (n_elements(vmark) NE 0) THEN BEGIN
      FOR k=1,nb_vmark DO BEGIN
        oplot,vbarx(k-1,*),vbary,psym=-3,linestyle=1
      ENDFOR
    ENDIF
;    string='('+strtrim(string(lval),2)+','+strtrim(string(bval),2)+')'
    IF (n_elements(coord) NE 0) THEN BEGIN
      string=' '
      IF i LE NNx-1 AND j LE NNy-1 THEN BEGIN
        IF coord EQ 'lb' OR coord EQ 'LB' THEN BEGIN
          lval=l(i,j)
          bval=b(i,j)
          string='(l,b)=('+strtrim(string(lval),2)+','+ $
                           strtrim(string(bval),2)+')'
        ENDIF
        IF coord EQ 'ad' OR coord EQ 'AD' THEN BEGIN
          nice_coord,a(i,j),d(i,j),aval,dval
          string=strtrim(string(aval),2)+','+strtrim(string(dval),2)
        ENDIF
        ens=[imax-imin,jmax-jmin]
        scale=min(ens)
        velmoy=minvel+(maxvel-minvel)/2.
        xyouts,velmoy,maxval-(maxval-minval)/10.,string, $
                charsize=char_size/scale
      ENDIF
    ENDIF
  ENDFOR
ENDFOR
!p.position(*)=0.
!p.charsize=savchar
!x.charsize=sxsize
!y.charsize=sysize
!x.tickname(*)=''
!y.tickname(*)=''

sortie:
END
