PRO mk_grid_val,h,xpos,ypos,vv,aa,nmx,nmy,l_grid,b_grid,gridx,gridy, $
		coox,cooy,grid=grid,silent=silent,no_x_label=no_x_label, $
		no_y_label=no_y_label,position_x=position_x, $
		position_y=position_y,ra_grid=ra_grid,dec_grid=dec_grid

;+
; NAME:
;          mk_grid_val
; PURPOSE:
;          Makes data to plot ticks and grid for image_cont
; CATEGORY:
;          Image Display
; CALLING SEQUENCE:
;          mk_grid_val,h,xpos,ypos,vv,aa,nmx,nmy,l_grid,b_grid,gridx,gridy, $
;		coox,cooy,grid=grid,silent=silent,no_x_label=no_x_label, $
;		no_y_label=no_y_label,position_x=position_x, $
;		position_y=position_y,ra_grid=ra_grid,dec_grid=dec_grid
; INPUTS:
;          h = input header
; OPTIONAL INPUTS:
;       position_x      = y ref position (low,high or middle) for x
;                         axis ticks
;       position_y      = x ref position (low,high or middle) for y
;                         axis ticks
; KEYWORD PARAMETERS:
;          None
; OUTPUTS:
;	  xpos,ypos	= position where ticks will be plotted (pixels)
;	  l_grid,b_grid	= position where ticks will be plotted (degrees)
;	  vv,aa		= labels for the ticks
;	  nmx, nmy	= number of intervals between ticks
;	  gridx,gridy	= ?
;	  coox,cooy	= input coordinates
; OPTIONAL OUTPUTS:
;          None
; COMMON BLOCKS:
;          @proj_common.com
; SIDE EFFECTS:
;          None
; RESTRICTIONS:
;          None
; PROCEDURE:
;          Uses create_struct 
; EXAMPLE:
;
; MODIFICATION HISTORY:
;          Created by J.-Ph. Bernard @ IAS on Wed Jul 30 1997
;          Modified JPB 3/02/00, to add formatting of vv and aa (for object plots).
;-

@proj_common.com

deltaxy=[1,1]

h2pix,h,/silent
dimen=[naxis1,naxis2]
cd=rota
crval=[crval1,crval2]
cdelt=[cdelt1,cdelt2]*!radeg
crpix=[crpix1,crpix2]
IF coord EQ 'RA' THEN RA='RA'

x_ax=1.*indgen(naxis1)+1
y_ax=1.*indgen(naxis2)+1

xsize = fix( dimen(0) )  &   ysize = fix( dimen(1) )
xsize1 = xsize-1 & ysize1 = ysize-1
IF NOT keyword_set(position_y) THEN BEGIN
  xmid=0					;default value
ENDIF ELSE BEGIN
  IF position_y EQ 'high' THEN xmid  = xsize1
  IF position_y EQ 'middle' THEN xmid  = xsize1/2.
  IF position_y EQ 'low' THEN xmid  = 0
ENDELSE
IF NOT keyword_set(position_x) THEN BEGIN
  ymid=0					;default value
ENDIF ELSE BEGIN
  IF position_x EQ 'high' THEN ymid  = ysize1
  IF position_x EQ 'middle' THEN ymid  = ysize1/2.
  IF position_x EQ 'low' THEN ymid  = 0
ENDELSE
;stop
;========== Defines the number of ticks
IF !x.ticks GT 0 THEN xtics=abs(!x.ticks) else xtics=8
IF !y.ticks GT 0 THEN ytics=abs(!y.ticks) else ytics=8

;========== Computes ticks values for the plot
;========== For projections where no simple expression is known, a numeric
;========== interpolation is performed
xedge=indgen(dimen(0)) & yedge=xedge & yedge(*)=ymid
pix2coo2,xedge,yedge,a,d			;coord of corners (deg)
;Check for ra=360 deg crossing
IF max(a)-min(a) NE ABS(a(0)-a(dimen(0)-1)) THEN BEGIN
  ind=where(a GE max([a(0),a(dimen(0)-1)]),count)
  IF count NE 0 THEN a(ind)=a(ind)-360.
ENDIF
a1=max(a) & a2=min(a)
yedge=indgen(dimen(1)) & xedge=yedge & xedge(*)=xmid
pix2coo2,xedge,yedge,a,d			;coord of corners (deg)
d1=min(d) & d2=max(d)
pixx = xsize/xtics                	;Number of X pixels between tic marks
pixy = ysize/ytics			;Number of Y pixels between tic marks
tics,a1,a2,xsize,pixx,raincr,RA=RA	;Find an even increment for RA
tics,d1,d2,ysize,pixy,decincr	;Find an even increment for Dec
tic_one,a1,pixx,raincr,botmin,xtic1,RA=RA	;Position of first RA tic
tic_one,d1,pixy,decincr,leftmin,ytic1	;Position of first Dec tic
nmx=fix((xsize1-xtic1-1)/pixx)			;Number of X tic marks
nmy=fix((ysize1-ytic1-1)/pixy)			;Number of Y tic marks
;print,nmx,nmy
IF coord EQ 'RA' THEN BEGIN
  IF not keyword_set(silent) THEN BEGIN
    Print,'x ticks sep=',raincr,' min'
    Print,'y ticks sep=',decincr,' arcmin'
  ENDIF
  coef_ra=4. & coef_dec=60.
ENDIF ELSE BEGIN
  IF not keyword_set(silent) THEN BEGIN
    Print,'x ticks sep=',raincr,' arcmin'
    Print,'y ticks sep=',decincr,' arcmin'
  ENDIF
  coef_ra=60. & coef_dec=60.
ENDELSE
ra_grid=(botmin+findgen(nmx+1)*raincr/coef_ra)/!RADEG		;in rad
dec_grid=(leftmin+findgen(nmy+1)*decincr/coef_dec)/!RADEG
l_grid=(botmin+findgen(nmx+1)*raincr/coef_ra)			;in deg
b_grid=(leftmin+findgen(nmy+1)*decincr/coef_dec)
my_ticlabels,botmin,nmx+1,raincr,vv,RA=RA,DELTA=deltaxy(0)
my_ticlabels,leftmin,nmy+1,decincr,aa,DELTA=deltaxy(1)
;=========== Filter labels to remove negative values
ind=where(strmid(strtrim(vv,2),0,1) EQ '-',count)
vall=360 & IF coord EQ 'RA' THEN vall=24
IF count NE 0 THEN BEGIN
  badstr=vv(ind)
  FOR i=0,count-1 DO BEGIN
    toto=str_sep(badstr(i),'!')
    toto(0)=toto(0)+vall
    badstr(i)=str_form(toto,'!')
  ENDFOR
  vv(ind)=badstr
ENDIF
easy_proj=['TAN']
ind=where(easy_proj EQ proj,count)
IF count NE 0 THEN BEGIN		;proj with inversible functions
  xpos=cons_coo(ra_grid,ymid,/lon)		;Line of constant RA
  ypos=cons_coo(dec_grid,xmid,/lat)		;Line of constant Dec
ENDIF ELSE BEGIN
;  ll_grid=l_grid & ind=where(l_grid LE 0,count)
;  IF count NE 0 THEN ll_grid(ind)=l_grid(ind)+360.
;  stop
  xedge=indgen(dimen(0)) & yedge=xedge & yedge(*)=ymid
  pix2coo2,xedge,yedge,coo1,coo2,/silent
;; modifs  done for FBB. May need to test for 360. deg instead
;  ind=where(coo1 GT 180.,count) & IF count NE 0 THEN coo1(ind)=coo1(ind)-360.
  ll_grid=l_grid
;;  ind=where(l_grid GT 180.,count)
;;  IF count NE 0 THEN ll_grid(ind)=ll_grid(ind)-360.
  xpos=interpol(xedge,coo1,ll_grid)
;  stop
  yedge=indgen(dimen(1)) & xedge=yedge & xedge(*)=xmid
  pix2coo2,xedge,yedge,coo1,coo2,/silent
  ypos=interpol(yedge,coo2,b_grid)
;  stop
ENDELSE
IF not keyword_set(silent) THEN BEGIN
  print,'Ra-ticks=' & print,ra_grid*!radeg
  print,'xpos=' & print,xpos
  print,'Ra-labels=' & print,vv
ENDIF

;========== Now do the grid stuff
IF keyword_set(grid) THEN BEGIN
  ind=where(easy_proj EQ proj,count)
  IF count NE 0 THEN BEGIN
;   Add to the grids values above and below center image range corresponding
;   to upper and lower side of the image
    xedge=indgen(dimen(0)) & yedge=xedge & yedge(*)=0
    pix2coo2,xedge,yedge,a,d,/silent			;coord of corners (deg)
;   Check for boundary crossing on a
    IF max(a)-min(a) NE ABS(a(0)-a(dimen(0)-1)) THEN BEGIN
      ind=where(a GE max([a(0),a(dimen(0)-1)]),count)
      IF count NE 0 THEN a(ind)=a(ind)-360.
    ENDIF
    adown=a
    yedge(*)=dimen(1)-1
    pix2coo2,xedge,yedge,a,d,/silent			;coord of corners (deg)
;   Check for boundary crossing on a
    IF max(a)-min(a) NE ABS(a(0)-a(dimen(0)-1)) THEN BEGIN
      ind=where(a GE max([a(0),a(dimen(0)-1)]),count)
      IF count NE 0 THEN a(ind)=a(ind)-360.
    ENDIF
    aup=a
    yedge=indgen(dimen(1)) & xedge=yedge & xedge(*)=0
    pix2coo2,xedge,yedge,a,d,/silent			;coord of corners (deg)
    dleft=d
    xedge(*)=dimen(0)-1
    pix2coo2,xedge,yedge,a,d,/silent			;coord of corners (deg)
    dright=d
    mia=min([aup,adown]) & maa=max([aup,adown])
    mid=min([dright,dleft]) & mad=max([dright,dleft])
    acur=l_grid(0) & nm=0
    WHILE acur LT maa DO BEGIN
      lastacur=acur
      acur=acur-raincr/coef_ra
      nm=nm+1
    ENDWHILE
    botmin=lastacur
    acur=l_grid(nmx) & np=0
    WHILE acur GT mia DO BEGIN
      acur=acur+raincr/coef_ra
      np=np+1
    ENDWHILE
    nmmx=nmx+nm+np-1
    dcur=b_grid(0) & nm=0
    IF decincr GT 0 THEN BEGIN
      lastdcur=dcur
      WHILE dcur LT mid DO BEGIN
        lastdcur=dcur
        dcur=dcur+decincr/coef_dec
        nm=nm+1
      ENDWHILE
    ENDIF ELSE BEGIN
      lastdcur=dcur
      WHILE dcur GT mid DO BEGIN
        lastdcur=dcur
        dcur=dcur-decincr/coef_dec
        nm=nm+1
      ENDWHILE
    ENDELSE
    leftmin=lastdcur
    dcur=b_grid(nmy) & np=0
    WHILE dcur LT mad DO BEGIN
      dcur=dcur+decincr/coef_dec
      np=np+1
    ENDWHILE
    nmmy=nmy+nm+np-1
    ra_grid=(botmin+findgen(nmmx+1)*raincr/4.)/!RADEG		;in rad
    dec_grid=(leftmin+findgen(nmmy+1)*decincr/60.)/!RADEG
    gridx=fltarr(Nmmx+1,dimen(1))
    gridy=fltarr(Nmmy+1,dimen(0))
    IF not keyword_set(silent) THEN BEGIN
      print,'Ra-grids=' & print,ra_grid*!radeg
    ENDIF
    yyvv=intarr(dimen(1))
    xxvv=intarr(dimen(0))
    FOR j=0,dimen(1)-1 DO BEGIN
      gridx(*,j)=cons_coo(ra_grid,j,/lon)
    ENDFOR
    FOR i=0,dimen(0)-1 DO BEGIN
      gridy(*,i)=cons_coo(dec_grid,i,/lat)
    ENDFOR
  ENDIF ELSE BEGIN				;The ones with no formula
    si=size(coox)
    IF si(0) NE 2 THEN BEGIN
      print,'creating 2D coordinate array'
      create_coo,h,coox,cooy,/silent
    ENDIF
  ENDELSE
ENDIF

;IF deltaxy(0) NE 1 THEN BEGIN
;  toto=l_grid & toto(*)=-32768. & j=0
;  FOR i=0,nmx,deltaxy(0) DO BEGIN
;    toto(j)=l_grid(i)
;    j=j+1
;  ENDFOR
;  ind=where(toto NE -32768.,count)
;  l_grid=toto(ind)
;ENDIF
si=size(l_grid) & nmx=si(1)-1 & vv=vv(0:nmx)
;IF deltaxy(1) NE 1 THEN BEGIN
;  toto=b_grid & toto(*)=-32768. & j=0
;  FOR i=0,nmy,deltaxy(1) DO BEGIN
;    toto(j)=b_grid(i)
;    j=j+1
;  ENDFOR
;  ind=where(toto NE -32768.,count)
;  b_grid=toto(ind)
;ENDIF
si=size(b_grid) & nmy=si(1)-1 & aa=aa(0:nmy)

IF nmx GT 30 THEN BEGIN
  fac=nmx/30+1
  new_nmx=round(nmx/fac)
  new_xpos=lonarr(new_nmx+1)
  new_vv=strarr(new_nmx+1)
  new_lgrid=fltarr(new_nmx+1)
  FOR i=0,new_nmx DO BEGIN
    new_xpos(i)=xpos(fac*i)
    new_vv(i)=vv(fac*i)
    new_lgrid(i)=l_grid(fac*i)
  ENDFOR
  vv=new_vv
  xpos=new_xpos
  l_grid=new_lgrid
  nmx=new_nmx
ENDIF
sieq=size(equinox)
IF sieq(1) EQ 7 THEN BEGIN
  equistr=strtrim(string(round(float(equinox))),2)
ENDIF ELSE BEGIN
  equistr=strtrim(string(round(equinox)),2)
ENDELSE

;Modif for the new IDL version
;Should be !version.release<'3.6.1a'
IF !version.release EQ '3.6.1a' $
   THEN BEGIN
  ntx=nmx+1 & nty=nmy+1		;number of ticks
  
  new_xpos=xpos
  xpos=fltarr(ntx+2)
  xpos(1:ntx)=new_xpos
  xpos(0)=0. & xpos(ntx+1)=dimen(0)
  
  new_ypos=ypos
  ypos=fltarr(nty+2)
  ypos(1:nty)=new_ypos
  ypos(0)=0. & ypos(nty+1)=dimen(1)
  
  new_vv=vv
  vv=strarr(ntx+2)
  vv(1:ntx)=new_vv
  vv(0)=' ' & vv(ntx+1)=' '
  
  new_aa=aa
  aa=strarr(nty+2)
  aa(1:nty)=new_aa
  aa(0)=' ' & aa(nty+1)=' '

  nmx=nmx+2 & nmy=nmy+2
  
ENDIF

IF !x.ticks EQ 0 THEN BEGIN             ;This is for automatic axis
  !x.ticks=nmx
  !x.tickname=strcompress(vv,/rem)
;  !x.tickname=vv
  !x.tickv=xpos
;  print,'xpos=',xpos
  IF not keyword_set(no_x_label) THEN BEGIN
    IF coord EQ 'GLON' THEN BEGIN
      !x.title='!6l (deg)'
    ENDIF
    IF coord EQ 'ELON' THEN BEGIN
      !x.title='!6eclon (deg)'
    ENDIF
    IF coord EQ 'RA' THEN BEGIN
      !x.title='!7a!6!d'+equistr+'!N (hr)'
    ENDIF
  ENDIF
  IF keyword_set(print) OR keyword_set(poscript) THEN !p.ticklen=-0.008
ENDIF ELSE BEGIN
  print,'Labeling is yours on x axis'
  IF keyword_set(grid) and not keyword_set(xgrid) THEN BEGIN
    l_grid=!x.tickv(0:!x.ticks)
  ENDIF
  !x.tickv=xpos
  !x.tickname='!6'+!x.tickname
  !x.tickname(!x.ticks+1:29)='' & !x.tickv(!x.ticks+1:29)=0
ENDELSE
IF !y.ticks EQ 0 THEN BEGIN
  !y.ticks=nmy
;  !y.tickname=aa
;stop
  !y.tickname=strcompress(aa,/rem)
  !y.tickv=ypos
  IF not keyword_set(no_y_label) THEN BEGIN
    IF coord EQ 'GLON' THEN BEGIN
      !y.title='!6b (deg)'
    ENDIF
    IF coord EQ 'ELON' THEN BEGIN
      !y.title='!6eclat (deg)'
    ENDIF
    IF coord EQ 'RA' THEN BEGIN
      !y.title='!7d!6!d'+equistr+'!N (degree)'
    ENDIF
  ENDIF
ENDIF ELSE BEGIN
  print,'Labeling is yours on y axis'
  IF keyword_set(grid) and not keyword_set(ygrid) THEN BEGIN
    b_grid=!y.tickv(0:!y.ticks)
  ENDIF
  !y.tickv=ypos
  !y.tickname='!6'+!y.tickname
  !y.tickname(!y.ticks+1:29)='' & !y.tickv(!y.ticks+1:29)=0
ENDELSE

vv='!6!N'+strcompress(vv)+'!A !N'
aa='!6!N'+strcompress(aa)+'!A !N'

END
