PRO my_BAR20,MIN=MIN,MAX=MAX,TYPE=TYPE,TITLE=TITLE,REV=REV,THICK=THICK, $
	   bar_size=bar_size,bar_pos=bar_pos,print=print, $
	   off_bar_pos=off_bar_pos,pcl=pcl,postcript=postcript,axis_color=axis_color

;+
;NAME:
;	my_bar20
;NAME/ONE LINE DESCRIPTION OF ROUTINE:
;     my_bar20 draws a color bar on the display.  
;
;CALLING SEQUENCE:  
;     my_bar20 [,MIN = Min] [,MAX = Max] [,TYPE = Type] $
;         [,TITLE = Title] [,/REV] [bar_size=], [/print],
;	  [[,bar_pos][,off_bar_pos=]]
;
;ARGUMENTS: (I = input, O = output, [] = optional)
;     Min    [I]   byt   Minimum color number in bar; default is 0
;     Max    [I]   byt   Maximum color number in bar; default is 255
;     Type   [I]   chr   Positioning indicator.
;                         TYPE='XT': X bar and label at top (default).
;                         TYPE='XB': X bar and label at bottom.
;                         TYPE='YR': Y bar and label at right.
;                         TYPE='YL': Y bar and label at left.
;     Title  [I]   chr   Bar title.
;     Rev    [I]   key   If specified, bar is reversed.
;
;WARNINGS:
;     Works only with devices that allow TV operations.
;     Only values between 0 and 255 are allowed for Min and Max;
;        Min must be less than Max if both are specified.
;
;EXAMPLES:
;     1.  Put a color bar from right to left at top of picture.
;                BAR
;     2.  Put a reversed color bar of width 20 pixels of colors 0 
;            through 150 at top of picture.
;                Bar,/rev, max = 150
;     3.  Put a reversed color bar at bottom of picture with title
;            'REVERSED'
;                Bar, /rev, type = 'xb', title='REVERSED'
;     4.  Put a color bar to left of picture 30 pixels wide with 
;             title 'STRAIGHT'
;                Bar, type = 'yl', title='STRAIGHT', thick=30
;     5.  Put a color bar to right of picture showing only colors 
;             50 through 200
;                Bar, type = 'yr', min = 50, max = 200
;#
;COMMON BLOCKS:
;     None
;
;PROCEDURE (AND OTHER PROGRAMMING NOTES):
;     Uses system variables to determine pixel size of window; scales
;     size of title, if any, to correspond to thickness of bar.
;
;PERTINENT ALGORITHMS, LIBRARY CALLS, ETC.:
;     NONE
;
;MODIFICATION HISTORY:
;   R. Sterner. 12 NOV, 1987.
;      Johns Hopkins University Applied Physics Laboratory.
;   RES  27 Jan, 1988 --- upgraded to Y bar and allowed labels.
;   Upgraded to Y bar; allowed labels; used keywords -- IDL Version II
;           Alice R. Trenholme, General Sciences Corporation 
;   Modified J.P.B. @ IPAC to add freedom to it
;.TITLE
;Routine BAR
;-
eco = 100.
IF not keyword_set(bar_pos) and not keyword_set(off_bar_pos) THEN BEGIN
  print,'One of bar_pos or off_bar_pos must be set'
  goto,sortie
ENDIF

toppcl=180.

save_pposition=!p.position
save_xticks=!x.ticks
save_yticks=!y.ticks
save_yminor=!y.minor
save_xminor=!x.minor
save_xtitle=!x.title
save_ytitle=!y.title
save_xtickname=!x.tickname
save_ytickname=!y.tickname
save_xticklen=!x.ticklen
save_yticklen=!y.ticklen

if !d.name eq 'TEK' or !d.name eq 'REGIS' then begin
  print, 'BAR will work only with X-windows or PostScript displays.'
endif else begin
  Bar_types = ['XT','XB','YL','YR']
  Bar_index = 0

  if n_elements(type) gt 0 then  BEGIN
    for i=0,3 do BEGIN
      if strupcase(type) eq bar_types(i) then bar_index = i
    ENDFOR
  ENDIF

  IF keyword_set(bar_pos) THEN BEGIN
    !p.position=bar_pos
  ENDIF ELSE BEGIN
 ;   print,!p.position
 ;   print,off_bar_pos
    toto=!p.position
    !p.position(0)=toto(0)+(toto(2)-toto(0)) $
				  *off_bar_pos(0)
    !p.position(1)=toto(1)+(toto(3)-toto(1)) $
				  *off_bar_pos(1)
    !p.position(2)=toto(0)+(toto(2)-toto(0)) $
				   *off_bar_pos(2)
    !p.position(3)=toto(1)+(toto(3)-toto(1)) $
				   *off_bar_pos(3)
    bar_pos=!p.position
  ENDELSE

  if not keyword_set(print)  then begin
     if !d.name eq 'PS' then eco=100. else eco=1.
     dx_vsize =  !d.x_vsize/eco	
     dy_vsize = !d.y_vsize/eco
  endif else begin
     dx_vsize =  640.
     dy_vsize = 512.
  endelse   
  
  ;print,'dx_vsize, dy_vsize=',dx_vsize,dy_vsize

  ;Define bar length in device units
  xlen=(bar_pos(2)-bar_pos(0))*dx_vsize
  ylen=(bar_pos(3)-bar_pos(1))*dy_vsize
    
  if keyword_set(rev) then rotation_ix=[1,1,2,2] $
                         else rotation_ix=[4,4,0,0]

  if bar_index ge 2 then begin 	
     bar_mat = bytscl(findgen(xlen,ylen),top=!d.n_colors-1)
  endif else begin   
     bar_mat = bytscl(findgen(ylen,xlen),top=!d.n_colors-1)
  endelse   


  IF keyword_set(MIN) THEN mii=min
  IF keyword_set(MAX) THEN maa=max
  if ((n_elements(mii) gt 0) or (n_elements(maa) gt 0)) then begin
    if n_elements(mii) eq 0 then mii = 0
    if n_elements(maa) eq 0 then maa = 255
   if (maa lt mii) then begin
     print,'  Disallowed values of minimum and/or maximum'
     return
   endif

  endif

  xx=findgen(100)/100.*(maa-mii)+mii
  babar=rotate(bar_mat,rotation_ix(bar_index))
;Used to set the position of the bar
;  stop
  contour,babar,/nodata,xstyle=5,ystyle=5,xtitle='',ytitle='',/noerase
   
  IF not keyword_set(print) and not keyword_set(postcript) THEN BEGIN
 ;   Print,'First TV type in my_bar20'
    tv,babar,0,0,/data
  ENDIF ELSE BEGIN
    posx=!p.position(0)*!d.x_size
    posy=!p.position(1)*!d.y_size
    IF not keyword_set(pcl) THEN BEGIN
      x_size=(!p.position(2)-!p.position(0))*!d.x_size
      y_size=(!p.position(3)-!p.position(1))*!d.y_size
;      Print,'Second TV type in my_bar20'
;      Print,'min max in babar = ',min(babar),max(babar)
;      Print,'!d.n_colors=',!d.n_colors
      tv,babar,posx,posy,xsize=x_size,ysize=y_size,/device
    ENDIF ELSE BEGIN
        ppos=[!p.position(0)*!d.x_size,!p.position(1)*!d.y_size, $
             !p.position(2)*!d.x_size,!p.position(3)*!d.y_size]
        ssx=ppos(2)-ppos(0)
        ssy=ppos(3)-ppos(1)
        sz=size(babar)
;        Print,'Third TV type in my_bar20'
        
       tv,poly_2d(bytscl(babar,min=0,max=255,top=toppcl), $
               [[0,0],[sz(1)/ssx,0]],[[0,sz(2)/ssy],[0,0]],0,ssx,ssy), $
               posx,posy,/device
    ENDELSE
  ENDELSE
;Displays the borders

  if bar_index ge 2 then begin
       init_yaxis
       !y.ticklen=-0.2
       !x.ticklen=0.
       !x.ticks=1 & !x.tickv=max(xx)
  endif else begin
       init_xaxis
       !x.ticklen=-0.2
       !y.ticklen=0.
       !y.ticks=1 & !y.tickv=max(xx)  
  
  endelse 
  
  !x.tickname(*)=' '
  
;  Print,'Bar title = ',title
  IF not keyword_set(pcl) THEN BEGIN
    plot,xx,xx,xtitle=' ',ytitle=' ',/nodata,/noerase,xstyle=1,ystyle=1, $
       title=title,color=axis_color
  ENDIF ELSE BEGIN
    plot,xx,xx,xtitle=' ',ytitle=' ',/nodata,/noerase,xstyle=1,ystyle=1, $
       pos=ppos,color=axis_color,/dev,title=title
  ENDELSE

endelse

!p.position=save_pposition
!x.ticks=save_xticks
!y.ticks=save_yticks
!y.minor=save_yminor
!x.minor=save_xminor
!x.title=save_xtitle
!y.title=save_ytitle
!x.tickname=save_xtickname
!y.tickname=save_ytickname
!x.ticklen=save_xticklen
!y.ticklen=save_yticklen
sortie:
end
