PRO clumping,input,Tt,mask,silent=silent,neighb=neighb

;+
; NAME:
;       clumping
; CALLING SEQUENCE:
;       clumping,input,Tt,mask[,/silent]
; PURPOSE:
;       Make a mask of individual connex clouds in a 2D or 3D cube
; INPUTS:
;       input = input image (2D or 3D)
;       Tt = Threshold value
; OPTIONAL INPUT:
;	neighb = a set (2 or 3) of values defining neighborhood
;	         default=[1,1] or [1,1,1]
; OUTPUTS:
;       mask = A mask giving positions of individual structures
; PROCEDURE AND SUBROUTINE USED
;	The positions where input > Tt are found and individual
;	clumps (pixels connected together) are identified and
;	labeled. mask contains 0s, except where indiv clumps
;	(labeled 1,2,...) have been found.
;	neighborhood is defined by neighb on each axis.
;	e.g neighb=[2,1,3] means that on x 2 pixels are neighbors if
;	the dist between them is <2, while on velocity axis neighbors
;	are <3 pixels appart.
; SIDE EFFECTS:
;       
; MODIFICATION HISTORY:
;       written by JP Bernard 25-Oct-92
;-

;on_error,2

IF N_PARAMS(0) LT 3 THEN begin
  print,'Calling Sequence: clumping,input,Tt,mask'
  print,'Accepted K-words: [,/silent][,neighb=]'
  goto,sortie
endif

si=size(input)
dim=si(0)
IF dim EQ 3 THEN BEGIN
  sx=si(1) & sy=si(2) & sz=si(3)
  sketch=intarr(sx,sy,sz)
  mask=intarr(sx,sy,sz)
  IF not keyword_set(neighb) THEN BEGIN
    xn=1 & yn=1 & vn=1
  ENDIF ELSE BEGIN
    neighb=round(neighb)
    xn=neighb(0) & yn=neighb(1) & vn=neighb(2)
  ENDELSE
ENDIF
IF dim EQ 2 THEN BEGIN
  sx=si(1) & sy=si(2)
  sketch=intarr(sx,sy)
  mask=intarr(sx,sy)
  IF not keyword_set(neighb) THEN BEGIN
    xn=1 & yn=1
  ENDIF ELSE BEGIN
    neighb=round(neighb)
    xn=neighb(0) & yn=neighb(1)
  ENDELSE
ENDIF

;find pixels satisfying threshold condition ==============================
indok=where(input GE Tt,countok)
if countok EQ 0 THEN begin
  print,'No data point satisfying Tt>',Tt
  goto,sortie
endif
IF n_elements(silent) EQ 0 THEN  BEGIN
  print,'clumping: I found ',countok,' points satisfying criteria'
ENDIF
sketch(indok)=1				;create the sketch
;Find individual clumps =========================================
IF n_elements(silent) EQ 0 THEN print,'Now finding indiv clumps'
remaining=where(sketch eq 1 and mask eq 0,others)
cloud_number=1
;delta=1.*dim			;2 for a 2-D Array, 3 for 3-D

IF (dim EQ 3) THEN BEGIN
  WHILE others NE 0 DO begin
    counter=0			;count number of points in each cloud
    coor=get_coor(remaining(0),sx,sy,sz)
    i0=coor(0)
    j0=coor(1)
    k0=coor(2)
    mask(i0,j0,k0)=cloud_number
    ring=1
    found=1
    WHILE found EQ 1 DO begin
      taille=2*ring+1
      debx=i0-ring*xn
      deby=j0-ring*yn
      debz=k0-ring*vn
      finx=i0+ring*xn
      finy=j0+ring*yn
      finz=k0+ring*vn
      IF debx LT 0 THEN debx=0
      IF deby LT 0 THEN deby=0
      IF debz LT 0 THEN debz=0
      IF finx GT sx-1 THEN finx=sx-1
      IF finy GT sy-1 THEN finy=sy-1
      IF finz GT sz-1 THEN finz=sz-1
      look_region=sketch(debx:finx,deby:finy,debz:finz)
      si_look=size(look_region)
      taille_x=si_look(1)
      taille_y=si_look(2)
      taille_z=si_look(3)
      out_region=mask(debx:finx,deby:finy,debz:finz)
      surounders=where((look_region EQ 1) AND (out_region EQ 0),suround)
      found=0			;indicate if connected points have been found
      IF suround NE 0 THEN begin
        si=size(surounders)
        taille=si(1)
        for c=1,taille do begin
          coor=get_coor(surounders(c-1),taille_x,taille_y,taille_z)
              i=coor(0)
              j=coor(1)
              k=coor(2)
          IF i0-ring*xn GT 0 THEN i=coor(0)+i0-ring*xn
          IF j0-ring*yn GT 0 THEN j=coor(1)+j0-ring*yn
          IF k0-ring*vn GT 0 THEN k=coor(2)+k0-ring*vn
          iinf=i-xn
          jinf=j-yn
          kinf=k-vn
          isup=i+xn
          jsup=j+yn
          ksup=k+vn
          if iinf lt 0 THEN iinf=0
          if jinf lt 0 THEN jinf=0
          if kinf lt 0 THEN kinf=0
          if isup gt sx-1 THEN isup=sx-1
          if jsup gt sy-1 THEN jsup=sy-1
          if ksup gt sz-1 THEN ksup=sz-1
          IF (total(mask(iinf:isup,jinf:jsup,kinf:ksup)) GT 0.) THEN begin
            mask(i,j,k)=cloud_number
            counter=counter+1
            found=1
          endif
        endfor
        ring=ring+1
      endif ELSE begin			;this is when a cloud as been found
;        cloud_number=cloud_number+1
;        g(0)=1
      endelse
    ENDWHILE
    cloud_number=cloud_number+1
    remaining=where(sketch EQ 1 and mask EQ 0,others)
  ENDWHILE
ENDIF

IF dim EQ 2 THEN BEGIN			;2D case
  WHILE others NE 0 DO begin
    counter=0			;count number of points in each cloud
    coor=get_coor(remaining(0),sx,sy)
    i0=coor(0)
    j0=coor(1)
    mask(i0,j0)=cloud_number
    ring=1
    found=1
    WHILE found EQ 1 DO begin
      taille_ring=2*ring+1
      debx=i0-ring
      deby=j0-ring
      finx=i0+ring
      finy=j0+ring
      IF debx LT 0 THEN debx=0
      IF deby LT 0 THEN deby=0
      IF finx GT sx-1 THEN finx=sx-1
      IF finy GT sy-1 THEN finy=sy-1
      look_region=sketch(debx:finx,deby:finy)
      out_region=mask(debx:finx,deby:finy)
      si_look=size(look_region)
      taille_x=si_look(1)
      taille_y=si_look(2)
      surounders=where((look_region EQ 1) AND (out_region EQ 0),suround)
      found=0			;indicate if connected points have been found
      IF suround NE 0 THEN begin
        si=size(surounders)
        taille=si(1)
        for c=1,taille do begin
          coor=get_coor(surounders(c-1),taille_x,taille_y)
	  i=coor(0)
	  j=coor(1)
          IF i0-ring GT 0 THEN i=coor(0)+i0-ring
          IF j0-ring GT 0 THEN j=coor(1)+j0-ring
          iinf=i-1
          jinf=j-1
          isup=i+1
          jsup=j+1
          if iinf lt 0 THEN iinf=0
          if jinf lt 0 THEN jinf=0
          if isup Gt sx-1 THEN isup=sx-1
          if jsup Gt sy-1 THEN jsup=sy-1
          IF (total(mask(iinf:isup,jinf:jsup)) GT 0.) THEN begin
            mask(i,j)=cloud_number
            counter=counter+1
            found=1
          endif
        endfor
        ring=ring+1
      endif ELSE begin			;this is when a cloud as been found
;        cloud_number=cloud_number+1
;        g(0)=1
      endelse
    ENDWHILE
    cloud_number=cloud_number+1
    remaining=where(sketch EQ 1 and mask EQ 0,others)    
  ENDWHILE
ENDIF
cloud_number=cloud_number-1
IF n_elements(silent) EQ 0 THEN BEGIN
  print,'Found a total of ',cloud_number,' individual clouds'
ENDIF

sortie:

END
