FUNCTION column_density_outflow,II,Tex,tau,line=line
;
; Look into Ji Yang Ph.D. thesis.
; written by Kazuhito DOBASHI 1998/06/05
;
;

;check inputs........................................
npar=N_PARAMS()
IF(npar LT 2) THEN BEGIN
print,'PURPOSE: To calculate molecular column density of outflows. '
print,'         Input the mean OPTICAL DEPTH of the high velocity component.'
print,'CALLING SEQUENCE: Ntot=column_density_outflow(II,Tex,tau,line=line)'
print,'INPUTS:'
print,'	II = Integrated intensity of outflow velocity component in Kkm/s'
print,'	Tex = Excitation temperature in K'
print,'	tau = Optical depth of the molecule in the outflow'
print,'	line = Select lines by the number. It will be 0 if not set.'
print,'		0.12CO J=1-0'
print,'		1.13CO J=1-0'
print,'		2.C18O J=1-0'
print,'		3.HCO+ J=1-0'
print,'		4.H13CO+ J=1-0'
print,'		5.HCN J=1-0'
print,'		6.CS J=1-0'
print,'		7.CS J=2-1'
print,'		8.C34S J=1-0'
print,'		9.C34S J=2-1'
print,'OUTPUTS:'
print,'	Ntot = Total column density of the molecule in cm^-2'
Ntot=-1
goto,ending
ENDIF
IF KEYWORD_SET(line) THEN number=line ELSE number=0
;undifined values------------------------------------
ind_good=where(II NE !indef AND Tex NE !indef AND tau NE !indef,cnt_good)
IF(cnt_good EQ 0) THEN GOTO, ending
Ntot=II & Ntot(*)=!indef
Jtex=Ntot
;General Constants.............................
h=6.6261*10.^(-27.)	;planck constant [erg s]
k=1.3807*10.^(-16.)	;boltzman constant [erg/K]
Tbb=2.7			;background radiation in K
pc=3.08*10^18.			;1 pc [cm]
au=1.50*10^13.			;1 AU [cm]
Mo=1.989*10^33.			;1 solar mass [g]
Mp=1.6735*10^(-24.)		;proton mass [g]
MH2=Mp*2.4			;mean molecular 
				; weight corrected for He[g]
;Molecular Constants............................
freq=dblarr(10)			;rest frequency
debye=fltarr(10)		;dipole moment
RJ=fltarr(10)			;level of rotational transition J+1-->J
freq(0)=115271.204*10.^(6.)	;12CO J=1-0
freq(1)=110201.370*10.^(6.)	;13CO J=1-0
freq(2)=109782.182*10.^(6.)	;C18O J=1-0
freq(3)=98188.523*10.^(6.)	;HCO+ J=1-0
freq(4)=86756.330*10.^(6.)	;H13CO+ J=1-0
freq(5)=88631.847*10.^(6.)	;HCN J=1-0
freq(6)=48990.964*10.^(6.)	;CS J=1-0
freq(7)=97980.968*10.^(6.)	;CS J=2-1
freq(8)=48206.956*10.^(6.)	;C34S J=1-0
freq(9)=96412.982*10.^(6.)	;C34S J=2-1
debye(0)=0.1098	;12CO J=1-0
debye(1)=0.1098	;13CO J=1-0
debye(2)=0.1098	;C18O J=1-0
debye(3)=4.48	;HCO+ J=1-0
debye(4)=4.48	;H13CO+ J=1-0	?
debye(5)=2.984	;HCN J=1-0
debye(6)=1.957	;CS J=1-0
debye(7)=1.957	;CS J=2-0	??
debye(8)=1.957	;C34S J=1-0
debye(9)=1.957	;C34S J=2-0	??
RJ(0)=0.	;12CO J=1-0
RJ(1)=0.	;13CO J=1-0
RJ(2)=0.	;C18O J=1-0
RJ(3)=0.	;HCO+ J=1-0
RJ(4)=0.	;H13CO+ J=1-0
RJ(5)=0.	;HCN J=1-0
RJ(6)=0.	;CS J=1-0
RJ(7)=1.	;CS J=2-1
RJ(8)=0.	;C34S J=1-0
RJ(9)=1.	;C34S J=2-1
;selection for molecular line.................
freq=freq(number)
debye=debye(number)
RJ=RJ(number)
;calculations............................
tm1=3*k*(RJ+1.)*10.^(16)
tm2=4*debye*debye*freq*!pi^3.*10.^(-11)
C0=tm1/tm2 & C0=C0*10.^(14)
T0=h*freq/k
Jtex(ind_good)=planck_kd(freq,Tex(ind_good))
Jbb=planck_kd(freq,Tbb)
;stop
fact1=Tex & fact1(*)=!indef
fact1(ind_good)=(exp(RJ*T0/2./Tex(ind_good)))/(RJ+1.)
ibeta=fact1
ibeta(ind_good)=tau/(1.-exp(-1.*tau(ind_good)))
fact2=fact1
fact2(ind_good)=ibeta(ind_good)/(Jtex(ind_good)-Jbb)
Ntot(ind_good)=C0/(1.-exp(-1.*T0/Tex(ind_good)))*$
	fact1(ind_good)*fact2(ind_good)*II(ind_good)$
	*Tex(ind_good) ;column density in cm-2
ending:
RETURN,Ntot
END
