FUNCTION excitation_temp,Tr,line=line
;
; Look into Ji Yang Ph.D. thesis.
;
;	written by Kazuhito DOBASHI 1998/06/05
;

;check inputs........................................
npar=N_PARAMS()
IF(npar LT 1) THEN BEGIN
print,'PURPOSE: To calculate the excitation temperature from molecular datas. '
print,'         Use only OPTICALLY THICK lines.'
print,'CALLING SEQUENCE: Tex=excitation_temp(Tr,line=line)'
print,'INPUTS:'
print,'	Tr = Gaussian fitted peak temperature in K'
print,'	line = Select lines by the number. It will be 0 if not set.'
print,'		0.12CO J=1-0'
print,'		1.13CO J=1-0'
print,'		2.C18O J=1-0'
print,'		3.HCO+ J=1-0'
print,'		4.H13CO+ J=1-0'
print,'		5.HCN J=1-0'
print,'		6.CS J=1-0'
print,'		7.CS J=2-1'
print,'		8.C34S J=1-0'
print,'		9.C34S J=2-1'
print,'OUTPUTS:'
print,'	Tex = Excitation temperature'
Tex=-1
goto,ending
ENDIF

IF KEYWORD_SET(line) THEN number=line ELSE number=0
;undefined values-----------------
ind_good=where(Tr NE !indef,cnt_good)
Tex=Tr
IF(cnt_good EQ 0) THEN goto, ending
;General Constants.............................
h=6.6261*10.^(-27.)	;plank constant [erg s]
k=1.3807*10.^(-16.)	;boltzman constant [erg/K]
Tbb=2.7			;background radiation in K
pc=3.08*10^18.			;1 pc [cm]
au=1.50*10^13.			;1 AU [cm]
Mo=1.989*10^33.			;1 solar mass [g]
Mp=1.6735*10^(-24.)		;proton mass [g]
MH2=Mp*2.4			;mean molecular 
				; weight corrected for He[g]
;Molecular Constants............................
freq=dblarr(10)			;rest frequency
debye=fltarr(10)		;dipole moment
freq(0)=115271.204*10.^(6.)	;12CO J=1-0
freq(1)=110201.370*10.^(6.)	;13CO J=1-0
freq(2)=109782.182*10.^(6.)	;C18O J=1-0
freq(3)=98188.523*10.^(6.)	;HCO+ J=1-0
freq(4)=86756.330*10.^(6.)	;H13CO+ J=1-0
freq(5)=88631.847*10.^(6.)	;HCN J=1-0
freq(6)=48990.964*10.^(6.)	;CS J=1-0
freq(7)=97980.968*10.^(6.)	;CS J=2-1
freq(8)=48206.956*10.^(6.)	;C34S J=1-0
freq(9)=96412.982*10.^(6.)	;C34S J=2-1
debye(0)=0.1098	;12CO J=1-0
debye(1)=0.1098	;13CO J=1-0
debye(2)=0.1098	;13CO J=1-0
debye(3)=4.48	;HCO+ J=1-0
debye(4)=4.48	;H13CO+ J=1-0	?
debye(5)=2.984	;HCN J=1-0
debye(6)=1.957	;CS J=1-0
debye(7)=1.957	;CS J=2-0	??
debye(8)=1.957	;C34S J=1-0
debye(9)=1.957	;C34S J=2-0	??

;selection for molecular line.................
freq=freq(number)
debye=debye(number)
;calculations............................
T0=h*freq/k
b1=T0/(exp(T0/Tbb)-1.)
Tex(ind_good)=T0/(alog(1.+T0/(Tr(ind_good)+b1)))

ending:
RETURN,Tex
END
