FUNCTION optical_depth_outflow,II,line,Tex,ratio=ratio
;
; 
; written by Kazuhito DOBASHI 1998/06/08
;
;

;check inputs........................................
npar=N_PARAMS()
IF(npar LT 3) THEN BEGIN
print,'PURPOSE: To calculate optical depth of outflow wings. '
print,'         Select very optically thick line and its isotope line with'
print,'         moderate optical depth (e.g., 12CO and 13CO) whose abundance'
print,'         ratio is known.'
print,'CALLING SEQUENCE: tau=optical_depth_outflow(II,line,Tex,ratio=ratio)'
print,'INPUTS:'
print,'	II = 1D array with 2 components. Input the integrated intensity'
print,'	     values in Kkm/s in the order of [optically thick,optically thin] lines.'
print,'	line = 1D array with 2 components. Select lines by the following numbers'
print,'	     in the order of [optically thick,optically thin] lines.'
print,'		0.12CO J=1-0'
print,'		1.13CO J=1-0'
print,'		2.C18O J=1-0'
print,'		3.HCO+ J=1-0'
print,'		4.H13CO+ J=1-0'
print,'		5.HCN J=1-0 : not used'
print,'		6.CS J=1-0'
print,'		7.CS J=2-1'
print,'		8.C34S J=1-0'
print,'		9.C34S J=2-1'
print,'----------------------------------------------------------------------'
print,'	Select the ISOTOPES with the SAME TRANSITION only.'
print,' Allowed line combinations;[0,1][0,2][1,2][3,4][6,8][7,9]'
print,'----------------------------------------------------------------------'
print,'Tex = Scalor. Excitation temperature in K'
print,'ratio = Scalor representing the abundance ratio: 89.9 for X(12CO)/X(13CO)'
print,'OUTPUTS:tau = 1D array with 2 components. Optical depths of the lines.'
tau=-1
goto,ending
ENDIF


;General Constants.............................
h=6.6261*10.^(-27.)	;planck constant [erg s]
k=1.3807*10.^(-16.)	;boltzman constant [erg/K]
Tbb=2.7			;background radiation in K
pc=3.08*10^18.			;1 pc [cm]
au=1.50*10^13.			;1 AU [cm]
Mo=1.989*10^33.			;1 solar mass [g]
Mp=1.6735*10^(-24.)		;proton mass [g]
MH2=Mp*2.4			;mean molecular 
				; weight corrected for He[g]
;Molecular Constants............................
freq=dblarr(10)			;rest frequency
debye=fltarr(10)		;dipole moment
isoratio=fltarr(10)			;level of rotational transition J+1-->J
freq(0)=115271.204*10.^(6.)	;12CO J=1-0
freq(1)=110201.370*10.^(6.)	;13CO J=1-0
freq(2)=109782.182*10.^(6.)	;C18O J=1-0
freq(3)=98188.523*10.^(6.)	;HCO+ J=1-0
freq(4)=86756.330*10.^(6.)	;H13CO+ J=1-0
freq(5)=88631.847*10.^(6.)	;HCN J=1-0
freq(6)=48990.964*10.^(6.)	;CS J=1-0
freq(7)=97980.968*10.^(6.)	;CS J=2-1
freq(8)=48206.956*10.^(6.)	;C34S J=1-0
freq(9)=96412.982*10.^(6.)	;C34S J=2-1
debye(0)=0.1098	;12CO J=1-0
debye(1)=0.1098	;13CO J=1-0
debye(2)=0.1098	;C18O J=1-0
debye(3)=4.48	;HCO+ J=1-0
debye(4)=4.48	;H13CO+ J=1-0	?
debye(5)=2.984	;HCN J=1-0
debye(6)=1.957	;CS J=1-0
debye(7)=1.957	;CS J=2-0	??
debye(8)=1.957	;C34S J=1-0
debye(9)=1.957	;C34S J=2-0	??
isoratio(0)=89.9	;12CO J=1-0
isoratio(1)=1.	;13CO J=1-0
isoratio(2)=0.002*89.9 ;C18O J=1-0
isoratio(3)=89.9	;HCO+ J=1-0
isoratio(4)=1.	;H13CO+ J=1-0
isoratio(5)=0.	;HCN J=1-0
isoratio(6)=95.02	;CS J=1-0
isoratio(7)=95.02	;CS J=2-1
isoratio(8)=4.21	;C34S J=1-0
isoratio(9)=4.21	;C34S J=2-1
;calculation.................
freq0=freq(line(0))
freq1=freq(line(1))
debye0=debye(line(0))
debye1=debye(line(1))
II0=II(0)
II1=II(1)
IF KEYWORD_SET(ratio) THEN ratio=ratio ELSE ratio=isoratio(line(0))/isoratio(line(1))
J0tex=planck_kd(freq0,Tex)
J1tex=planck_kd(freq1,Tex)
J0bb=planck_kd(freq0,Tbb)
J1bb=planck_kd(freq1,Tbb)
tmp1=(J0tex-J0bb)*II1/(J1tex-J1bb)/II0
tau1=-1.*alog(1.-tmp1)
tau0=ratio*tau1
tau=[tau0,tau1]
ending:
RETURN,tau
END
