FUNCTION toh2, nx,sw
;
;INPUTS:
;	nx = column density of a given kind of molecules [cm-2]
;	sw= switch to select molecules and conversion factor
; OUTPUT:
;   H2 column density is returned.
;REFERENCES:
; ----from my thesis (p.193) + others
;	1. Dickman, R.L. 1978,ApJS,37,407
;	2. Frerking, M. A., Langer, W.D., & Wilson, R.W 1982, ApJ,262,590
;	3. Duvert, G., Cernicharo, J. & Baudry, A., 1986, A&A, 164,349
;	4. Cernicharo, J., & Guelin,M. 1987, A&A, 176,299
;	5. Bachiller, R., & Cernicharo, J. 1986, A&A, 166, 283
;	6. Linke,R.A.,& Goldsmith, P.F. 1980,ApJ, 235,437
;	7. Tatematsu, K., Umemoto,T., Murata, Y., & Chen, H. 1993, ApJ, 419, 746
;	8. Yang, J. 1990, PhD thesis, p.5-9 and references therein
;	9. Bernard et al. 1998 in preparation. (Measured in NCS1)
;	written by DOBASHI 1998/6/8


;print usage---------------------------------------------------------
npar=N_PARAMS()
IF(npar LT 2) THEN BEGIN
print,'PURPOSE: To calculate H2 column density from other molecules.'
print,'CALLING SEQUENCE: NH2=toh2(nx,sw)'
print,'INPUTS:'
print,'	nx = column density of a certain molecule'
print,'	sw = switch to select the molecule'
print,'	      which should be an integer (1-12) as follows;'
print,'		1.13CO-->H2'
print,'		2.C18O-->H2(5<Av<11)'
print,'		3.C18O-->H2(4<Av<15)'
print,'		4.C18O-->H2(1<Av<7)'
print,'		5.C18O-->H2(2<Av<6)'
print,'		6.C18O-->H2(0.6<Av<1.5)'
print,'		7.12CO-->H2 '
print,'		8.CS  -->H2(in normal cloud)'
print,'		9.CS  -->H2(in outflow)'
print,'	       10.C34S-->H2(in normal cloud) should be corrected later'
print,'	       11.C34S-->H2(in outflow) should be corrected later'
print,'	       12.HCO+-->H2'
print,'	       13.H13CO+->H2'
print,'	       14.CS-->H2(NCS1 outer region)'
print,'	       15.CS-->H2(NCS1 center region)'
print,'	       16.CS-->H2(NCS1 outflow)'
print,'	       17.C34S-->H2(NCS1 outer region)'
print,'	       18.C34S-->H2(NCS1 center region)'
print,'	       19.H13CO+-->H2(NCS1 outer region)'
print,'	       20.H13CO+-->H2(NCS1 center region)'
print,'OUTPUTS:'
print,'	NH2 = H2 column density'
Ntot=-1
goto,ending
ENDIF

;IF KEYWORD_SET(line) THEN number=line ELSE number=1
;undifined values------------------------------------
ind_good=where(nx NE !indef,cnt_good)
IF(cnt_good EQ 0) THEN GOTO, ending
nh2=nx & nh2(*)=!indef
;calculation------------------------------------------------------
CASE sw of
;from 13CO to H2..................
1: nh2(ind_good)=nx(ind_good)*5*10.^5.	;ref.1
;from C18O to H2...................
2:nh2(ind_good)=(nx(ind_good)/(2.6*10.^14.)+2.7)*10.^21. ; ref.2 (5<Av<11)
3:nh2(ind_good)=(nx(ind_good)/(1.8*10.^14.)+3.7)*10.^21. ; ref.2 (4<Av<15)
4:nh2(ind_good)=(nx(ind_good)/(2.7*10.^14.)+1.0)*10.^21. ; ref.3 (1<Av<7)
5:nh2(ind_good)=(nx(ind_good)/(2.7*10.^14.)+1.5)*10.^21	;ref.4 (2<Av<6)
6:nh2(ind_good)=nx(ind_good)*2.8*10.^6.		 ;ref.5(0.6<Av<1.5)
;from 12CO to H2................
7:nh2(ind_good)=nx(ind_good)*1*10.^4	;ref.2
;from CS to H2.................
8:nh2(ind_good)=nx(ind_good)*1.*10.^9	; ref.6 (in normal clouds)
9:nh2(ind_good)=nx(ind_good)*1.*10.^8	;ref.7 (in outflow lobes)
;from C34S to H2.................
10:nh2(ind_good)=nx(ind_good)*1.*95.02/4.21*10.^9 ; ref.6  (in normal clouds) 
11:nh2(ind_good)=nx(ind_good)*1.*95.02/4.21*10.^8 ;ref.7 (in outflow lobes)
;from HCO+ to H2.............
12:nh2(ind_good)=nx(ind_good)/(2.5*10.^(-9.))	;ref.8
;from H13CO+ to H2.............
13:nh2(ind_good)=nx(ind_good)/(2.5*10.^(-9.))*89 ;ref.8 + 13C:12CO=1:89
;NCS1 45m data (ref.9)..................
14:nh2(ind_good)=nx(ind_good)/(6.2*10.^(-9.))	;CS, NCS1 quiescent
15:nh2(ind_good)=nx(ind_good)/(1.3*10.^(-8.))	;CS, NCS1 center
16:nh2(ind_good)=nx(ind_good)/(2.0*10.^(-8.))	;CS, NCS1 outflow
17:nh2(ind_good)=nx(ind_good)/(6.8*10.^(-10.))	;C4S, NCS1 quiescent cloud
18:nh2(ind_good)=nx(ind_good)/(1.1*10.^(-9.))	;C34S, NCS1 center
19:nh2(ind_good)=nx(ind_good)/(1.2*10.^(-10.))	;H13CO+, NCS1 quiescent cloud
20:nh2(ind_good)=nx(ind_good)/(2.7*10.^(-10.))	;H13CO+, NCS1 center

ELSE: print,'input munber from 1-20'
ENDCASE
ending:
return, nh2
end