FUNCTION cut_image,im,hin,hout=hout,silent=silent

;+
; NAME:
;       cut_image
; CALLING SEQUENCE:
;       imout=cut_image(im,hin[,hout=hout][,/silent])
; PURPOSE:
;    cut a fits image to trim undefined lines and columns
; INPUTS:
;      im,hin = image (2D) and header
; OPTIONAL INPUT:
;      silent : If set, behaves silently
; OUTPUTS:
;       imout = trimmed image
; OPTIONAL OUTPUTS:
;      hout = header for imout
; PROCEDURE AND SUBROUTINE USED
;       Straightforward
;       Uses sxpar.pro
;            find_ind.pro
; RESTRICTIONS
; MODIFICATION HISTORY:
;       written JPB 15-08-92
;-

IF N_PARAMS(0) LT 2 THEN BEGIN
  print,'Calling Sequence: imout=cut_image(im,hin[,hout=hout][,/silent])'
  goto,sortie
ENDIF

si=size(im) & Nx=si(1) & Ny=si(2)

indef_line=intarr(Nx)
indef_col=intarr(Ny)

;=== number of !indef values per line and columns
FOR i=0,Nx-1 DO BEGIN
  ind=where(im(i,*) EQ !indef,count)
  indef_line(i)=count
ENDFOR
FOR j=0,Ny-1 DO BEGIN
  ind=where(im(*,j) EQ !indef,count)
  indef_col(j)=count
ENDFOR

;=== compute where the image must be cut
what=Ny
df_line=find_ind(indef_line,what,/silent,count=c_line)
what=Nx
df_col=find_ind(indef_col,what,/silent,count=c_col)

ifrom=[0] & jfrom=[0]
ito=[Nx-1] & jto=[Ny-1]

IF c_line NE 0 THEN BEGIN
  ind=where(df_line(*,0) EQ 0,count)
  IF count NE 0 THEN ifrom=reform(df_line(ind,1))+1
  ind=where(df_line(*,1) EQ Nx-1,count)
  IF count NE 0 THEN ito=reform(df_line(ind,0))-1
ENDIF
IF c_col NE 0 THEN BEGIN
  ind=where(df_col(*,0) EQ 0,count)
  IF count NE 0 THEN jfrom=reform(df_col(ind,1))+1
  ind=where(df_col(*,1) EQ Ny-1,count)
  IF count NE 0 THEN jto=reform(df_col(ind,0))-1
ENDIF
ifrom=ifrom(0) & jfrom=jfrom(0)
ito=ito(0) & jto=jto(0)

IF not keyword_set(silent) THEN BEGIN
  print,'image will be cut in i from ',ifrom,' to',ito
  print,'image will be cut in j from ',jfrom,' to',jto
ENDIF

imout=im(ifrom:ito,jfrom:jto)

;update the header accordingly
;IF keyword_set(hout) THEN BEGIN
;  IF keyword_set(hin) THEN BEGIN
    hout=hin
    sxaddpar,hout,'NAXIS1',ito-ifrom+1
    sxaddpar,hout,'NAXIS2',jto-jfrom+1
    sxaddpar,hout,'CRPIX1',sxpar(hin,'CRPIX1')-ifrom
    sxaddpar,hout,'CRPIX2',sxpar(hin,'CRPIX2')-jfrom
;  ENDIF ELSE BEGIN
;    print,'hin must be set when using hout'
;  ENDELSE
;ENDIF

RETURN,imout
sortie:
END
