FUNCTION enlarge,ind,Npix,sx,sy

;+
; NAME:
;       enlarge
; CALLING SEQUENCE:
;       ind2=enlarge(ind,Npix,sx,sy)
; PURPOSE:
;       enlarge a region pointed by ind in a 2D array by Npix pixels
;	  the area selected is enlarged by neighbor pixels
; INPUTS:
;       ind = index of the region
;	Npix = number of pixels to extend
;	sx, sy = size of the array in x and y
; OPTIONAL INPUT:
;
; OUTPUTS:
;       modified index
; SUBROUTINE AND PROCEDURE USED
;
; SIDE EFFECTS:
;	Should use long loop index, instead of cutting loops.
; MODIFICATION HISTORY:
;       WRITTEN, J.P. Bernard, 6-11-1992
;	modified JPB Jun 21 1994 to deal with large indexes sets
;-


if n_params() eq 0 then begin
  print,'Calling sequence: ind2=enlarge(ind,Npix,sx,sy)'
  goto,sortie
endif

si=size(ind)
sind=si(1)
work=intarr(sx,sy)
work(ind)=1

;coor=get_coor(ind,sx,sy)
coor=index2ij(ind,[sx,sy])

IF sind-1 LT 30000 THEN BEGIN
  FOR i=0,sind-1 DO BEGIN
    ipix=coor(i,0)
    jpix=coor(i,1)
    imax=ipix+Npix
    jmax=jpix+Npix
    imin=ipix-Npix
    jmin=jpix-Npix
    IF imin LT 0 THEN imin=0
    IF jmin LT 0 THEN jmin=0
    IF imax GT sx-1 THEN imax=sx-1
    IF jmax GT sy-1 THEN jmax=sy-1
    work(imin:imax,jmin:jmax)=1
  ENDFOR
ENDIF ELSE BEGIN
  nloop=(sind-1)/30000       ;number of 30000 loops
  FOR loop=0l,nloop-1 DO BEGIN
    FOR ii=0,30000-1 DO BEGIN
      i=long(ii)+loop*30000
      ipix=coor(i,0)
      jpix=coor(i,1)
      imax=ipix+Npix
      jmax=jpix+Npix
      imin=ipix-Npix
      jmin=jpix-Npix
      IF imin LT 0 THEN imin=0
      IF jmin LT 0 THEN jmin=0
      IF imax GT sx-1 THEN imax=sx-1
      IF  jmax GT sy-1 THEN jmax=sy-1
      work(imin:imax,jmin:jmax)=1
    ENDFOR
  ENDFOR
  remaining=sind-1-nloop*30000
  FOR ii=0,remaining-1 DO BEGIN
    i=long(ii)+nloop*30000
    ipix=coor(i,0)
    jpix=coor(i,1)
    imax=ipix+Npix
    jmax=jpix+Npix
    imin=ipix-Npix
    jmin=jpix-Npix
    IF imin LT 0 THEN imin=0
    IF jmin LT 0 THEN jmin=0
    IF imax GT sx-1 THEN imax=sx-1
    IF jmax GT sy-1 THEN jmax=sy-1
    work(imin:imax,jmin:jmax)=1
  ENDFOR
ENDELSE

indout=where(work EQ 1)

return,indout

sortie:

END
