FUNCTION enlarge_image,d,h,enlarge_size,hout,round=round

;+
; NAME:
;     enlarge_image
; PURPOSE
;     enlarges an image/header
; CATEGORY:
;     Image Processing
; CALLING SEQUENCE:
;     imout=enlarge_image(im,h,enlarge_size,hout)
; INPUTS:
; im             = image data
; h              = data header
; enlarge_size   = size to be enlarged (degree)
; OPTIONAL INPUT PARAMETERS:
; None
; OUTPUTS:
;     imout      = enlarged image
; OPTIONAL OUTPUT PARAMETERS:
;     hout       = enlarged header
; ACCEPTED KEY-WORDS:
;     None
; COMMON BLOCKS:
;     None
; SIDE EFFECTS:
;     None
; RESTRICTIONS:
; PROCEDURE:
;     
; EXAMPLES
;    
; MODIFICATION HISTORY:
;	Written by J.P. Bernard 11/04/2001
;-

IF N_PARAMS(0) EQ 0 THEN BEGIN
    print,'Calling Sequence:'
    print,'imout=enlarge_image(im,h,enlarge_size,hout)'
    goto,sortie
ENDIF

crpix1=sxpar(h,'CRPIX1') & crpix2=sxpar(h,'CRPIX2')
cdelt1=sxpar(h,'CDELT1') & cdelt2=sxpar(h,'CDELT2')
Npix1=sxpar(h,'NAXIS1') & Npix2=sxpar(h,'NAXIS2')

dNx=abs(round(enlarge_size(0)/cdelt1))
dNy=abs(round(enlarge_size(1)/cdelt2))

NNpix1=Npix1+2*dNx
NNpix2=Npix2+2*dNy

Ncrpix1=crpix1+dNx
Ncrpix2=crpix2+dNy

hout=h

sxaddpar,hout,'CRPIX1',Ncrpix1 & sxaddpar,hout,'CRPIX2',Ncrpix2
sxaddpar,hout,'NAXIS1',NNpix1 & sxaddpar,hout,'NAXIS2',NNpix2

;stop
create_coo2,hout,coo1,coo2,/silent
h2pix,h,/silent
coo2pix2,coo1,coo2,x,y,/silent
x=round(x) & y=round(y)
dd=my_bilinear2(d,x,y,/no_remove_indef)
;dd=bilinear(d,x,y)

;dd=project2(h,d,hout,/silent,round=round)

RETURN,dd
sortie:
END
