PRO gaussfit_n, x, a, f, pder
;+
;PURPOSE:
;	To usr under the "curvefit" function.	
;	To fit the given array with N gaussians, and to return the 
;	best fitting gaussian function. The function is 
;	written as follows;
;	  y=SIGMA[A(3*i)exp(-((x-A(3*i+1))/sqrt(2.)/A(3*i+2))^2)] where A(i) is
;	coefficients.
;
; CALLING SEQUENCE:
;	Result = CURVEFIT(X,Y,W,A,FUNCTION_NAME='gaussfit_1')
;
;		
;INPUT:
;	X		=	1 D array. X axis.		
;	Y		=	1D array to fit. Y axis.
;	W		=	1D array for the fitting weight.
;	A		=	1D array. The First estimate for 
;				 the coefficients.
;	FUNCTION_NAME	=	This function.	
;OUTPUT:	
;	Result		=	1D array (gaussian function) best fitting
;				the raw data.
;	A		=	1D array for the best fitting.
; MODIFICATION HISTORY:
;	Written by Kazuhito Dobashi Feb-97
;-

on_error,2	;return to caller

; Check N_param================
if N_params() LT 1 then begin	
      print,'Use this under "curvefit" function!'	
      print,"Usage - Result = CURVEFIT(X,Y,W,A,FUNCTION_NAME='gaussfit_N')"
      print,'Input - X(x data),Y(y data),W(weight),A(coefficients)'
      print,'Output - Result,A (1D array),'
      print,'A(i) - y(i)=A(3*i+i)exp(-((x-A(3*i+1))/sqrt(2)/A(3*i+2))^2)'
     goto,sortie
  endif

; start from here================

Ncomp=N_ELEMENTS(a) & Ncomp=Ncomp/3L	;number of components
bx=x & bx=0.
f=bx

FOR i=0,Ncomp-1 DO BEGIN
bx=(x-a(3*i+1))/sqrt(2.)/a(3*i+2)
f=f+a(3*i)*exp(-bx*bx)
ENDFOR

IF N_PARAMS() GE 4 THEN BEGIN 
pder=fltarr(N_ELEMENTS(x),Ncomp*3)
FOR i=0,Ncomp-1 DO BEGIN
 ff=x
 bx=(x-a(3*i+1))/sqrt(2.)/a(3*i+2)
 ff=a(3*i)*exp(-bx*bx)
 pder(*,3*i)=exp(-bx*bx)
 pder(*,3*i+1)=ff*bx/sqrt(2.)/a(3*i+2)
 pder(*,3*i+2)=ff*bx*bx/a(3*i+2)
END
ENDIF
sortie:
END
