FUNCTION gaussian_fit,xdata,ydata,coefficient=initA,$
                      vrange=vrange,trange=trange,n_component=n_component,$
                      weight=weight,nonstop=nonstop,to_be_fit=to_be_fit, $
                      niter=niter,chisqr=chisqr,silent=silent, $
                      baseline_vel=baseline_vel,baseline_degree=baseline_degree

;on_error,2
;initial setting============================
vel13=xdata	;x data

IF keyword_set(baseline_vel) AND keyword_set(baseline_degree) THEN BEGIN
  Print,'Removing baseline degree ...',baseline_degree
  Nvel=(size(baseline_vel))(1)
  weight=vel13 & weight(*)=0.
  FOR i=0,Nvel-1 DO BEGIN
    ind=where(vel13 GE min([baseline_vel(i,0),baseline_vel(i,1)]) $
             AND vel13 LE max([baseline_vel(i,0),baseline_vel(i,1)]),count)
     IF count NE 0 THEN BEGIN
       weight(ind)=1
     ENDIF        
  ENDFOR
  coef_base=polyfitw(vel13,ydata,weight,baseline_degree)
  dat=ydata-poly(vel13,coef_base)
ENDIF ELSE BEGIN
  dat=ydata	;y data
ENDELSE

Ncmp=100l	;number of gaussian comp (initail)
loadct,13	;color table 13
IF NOT keyword_set(nonstop) THEN BEGIN ; nonstop mode (NE 0)
   print,'Interactive mode....'
ENDIF ELSE BEGIN
   IF((N_ELEMENTS(initA) EQ 0) OR $
      (N_ELEMENTS(vrange) EQ 0) OR $
      (N_ELEMENTS(trange) EQ 0) OR $
      (N_ELEMENTS(n_component) EQ 0) OR $
      (N_ELEMENTS(weight) EQ 0)) THEN BEGIN
      print,'Set all the parameters for "NONSTOP" mode.'
      yfit=-1L
      goto,endingline
   ENDIF
   Ncmp=n_component
   GOTO,fitting
ENDELSE


IF(KEYWORD_SET(vrange) AND KEYWORD_SET(trange)) THEN BEGIN 
 ;display the spectrum
 trange=trange(sort(trange)) ;display x range
 vrange=vrange(sort(vrange)) ;display y range
 plot,vel13,dat,psym=10,xr=vrange,yr=trange,/ysty,/xsty
ENDIF ELSE BEGIN
 plot,vel13,dat,psym=10
 print,'Click display range (X - min and max)'
 CURSOR,xmin,y,/data,/down
 CURSOR,xmax,y,/data,/down
 vrange=[xmin,xmax]
 vrange=vrange(sort(vrange))
 print,'Click display range (Y - min and max)'
 CURSOR,x,ymin,/data,/down
 CURSOR,x,ymax,/data,/down
 trange=[ymin,ymax]
 trange=trange(sort(trange))
ENDELSE

plot,vel13,dat,xr=vrange,yr=trange,/xsty,/ysty,psym=10

IF KEYWORD_SET(n_component) THEN BEGIN ; number of gaussian component
 Ncmp=n_component
ENDIF ELSE BEGIN
 print,'Input number of components'
 read,Ncmp
 Ncmp=LONG(Ncmp)
ENDELSE

IF KEYWORD_SET(weight) THEN BEGIN
  weight=weight
ENDIF ELSE BEGIN
  print,'I will use a flat weight'
  weight=dat & weight(*)=1.
ENDELSE

;set initial peaks (Tr and Vlsr) =====================================
initA=fltarr(Ncmp*3) & initA(*)=0.
FOR i=0,Ncmp-1 DO initA(3*i+2)=0.5
x=0. & y=x
counter=0
FOR i=0l,Ncmp-1 DO BEGIN
;printing--------------------
 IF (i EQ 0) THEN BEGIN
    print,'Click the ',strcompress(string(i+1),/remove_all),$
	' st peak position (Tr and Vel)'
 ENDIF
 IF(i EQ 1) THEN BEGIN
    print,'Click the ',strcompress(string(i+1),/remove_all),$
	' nd peak position (Tr and Vel)'
 ENDIF
 IF(i GT 1) THEN BEGIN
    print,'Click the ',strcompress(string(i+1),/remove_all),$
	' th peak position (Tr and Vel)'
 ENDIF
print,'..Click the same position to exit..'
;read peaks----------------------
xpre=x & ypre=y
cursor,x,y,/data,/down	;read peak pos
IF(x EQ xpre AND y EQ ypre) THEN GOTO,sortie
 print,'Vel=',strcompress(string(x),/remove_all),$
	'   Tr=',strcompress(string(y),/remove_all)
 initA(3*i)=y & initA(3*i+1)=x
;see the results--------------------
ff=generate_gaussian(Ncmp,vel13,initA)
plot,vel13,dat,xr=vrange,/xsty,psym=10,xtitle='Vlsr km/s',$
ytitle='Tr* K',yr=trange,/ysty
oplot,vel13,ff,color=120
  FOR j=0,i DO BEGIN
   f=generate_gaussian(1,vel13,initA(3*j:3*j+2))
   oplot,vel13,f,color=100
  ENDFOR
counter=counter+1
ENDFOR
sortie:Ncmp=counter
;more precise setting (dV)================
x=fltarr(3*Ncmp) & y=x
loop1:
print,'Click and drag points for precise corection.'
print,'To exit, just click mouse bottom without moving.'
FOR i=0,Ncmp-1 DO BEGIN
x(3*i)=initA(3*i+1)
y(3*i)=initA(3*i)
ddv=initA(3*i+2)
x(3*i+1)=x(3*i)-ddv
x(3*i+2)=x(3*i)+ddv
y(3*i+1)=initA(3*i)*exp(-((x(3*i+2)-x(3*i))/sqrt(2.)/ddv)^2)
y(3*i+2)=y(3*i+1)
ENDFOR

ff=generate_gaussian(Ncmp,vel13,initA)
plot,vel13,dat,xr=vrange,/xsty,psym=10,xtitle='Vlsr km/s',yr=trange,/ysty,$
ytitle='Tr* K',title='Initial setting for the gaussian fitting'
oplot,vel13,ff,color=120
  FOR j=0,Ncmp-1 DO BEGIN
   f=generate_gaussian(1,vel13,initA(3*j:3*j+2))
   oplot,vel13,f,color=100
  ENDFOR
oplot,x,y,psym=6,color=130

ddx=(vrange(1)-vrange(0))/20.
ddy=(trange(1)-trange(0))/20.
xpos=vrange(0)+ddx
ypos=trange(1)-ddy
xyouts,xpos,ypos,'Tr*        Vlsr        dV',/DATA,color=100,SIZE=1.5
  FOR j=0,Ncmp-1 DO BEGIN
   ypos=ypos-ddy
    FOR k=0,3-1 DO BEGIN
      charac=strcompress(string(initA(3*j+k)),/remove_all)
      charac=strmid(charac,0,6)
      xyouts,xpos,ypos,charac,/DATA,color=100,SIZE=1.5
      xpos=xpos+ddx*3.
    ENDFOR
   xpos=xpos-ddx*9.
  ENDFOR

loop2: CURSOR,orix,oriy,/data,/down
CURSOR,newx,newy,/data,/up
IF(orix EQ newx AND oriy EQ newy) THEN GOTO,sortie1
dis=(x-orix)^2.+(y-oriy)^2.
ind=where(dis EQ min(dis),cnt)
 IF(cnt NE 1) THEN BEGIN
  print,'Click again...'
  goto,loop2
 ENDIF

judge=long(3.*((ind(0)+0.0)/3.-ind(0)/3)+0.01)

IF(judge EQ 0) THEN BEGIN
  initA(ind(0)+1)=newx & initA(ind(0))=newy
ENDIF 
IF(judge EQ 1) THEN initA(ind(0)+1)=abs(newx-initA(ind(0)))
IF(judge EQ 2) THEN initA(ind(0))=abs(newx-initA(ind(0)-1))
goto,loop1
sortie1:

;least square fitting==================================
fitting:
IF keyword_set(vrange) THEN BEGIN
  ind=where(vel13 GT min(vrange) AND vel13 LT max(vrange),count)
  vel=vel13(ind)
  spec=dat(ind)
ENDIF ELSE BEGIN
  vel=vel13
  spec=dat
ENDELSE
weight=dat & weight(*)=1.

print,'Trying the least square fitting.....'
IF(Ncmp GT 3) THEN print,'It will take some time...'
;Modif JPB to use my_curvefit
;yfit=curvefit(vel13,dat,weight,initA,function_name='gaussfit_n')
sigA=fltarr(3*Ncmp) & sigA(*)=!indef
yfit=my_curvefit(vel,spec,weight,initA,sigA,fail, $
                 function_name='gaussfit_n',to_be_fit=to_be_fit, $
                 niter=niter,chisqr=chisqr,silent=silent)
print,'chisqr=',chisqr
print,'niter=',niter
print,'fail=',fail
;stop
all_sig=strarr(3*Ncmp) & all_sig(*)='??'
help,all_sig(where(to_be_fit NE 0))
help,string(strtrim(sigA,2),format='(A6)')
si1=size(all_sig(where(to_be_fit NE 0))) & si1=si1(1)
si2=size(string(strtrim(sigA,2),format='(A6)')) & si2=si2(1)
IF si1 EQ si2 THEN BEGIN
  all_sig(where(to_be_fit NE 0))=string(strtrim(sigA,2),format='(A6)')
ENDIF ELSE BEGIN
  all_sig=string(strtrim(sigA,2),format='(A6)')
ENDELSE

frmt='(A4,F7.3,A2,A6,A7,F7.3,A2,A6,A7,F7.3,A2,A6)'
FOR i=0,Ncmp-1 DO BEGIN
  print,'Tr=',initA(3*i),'+-',all_sig(3*i),'V =',initA(3*i+1),'+-', $
              all_sig(3*i+1),'dV=',initA(3*i+2),'+-',all_sig(3*i+2),format=frmt
ENDFOR

;display the final results====================
ff=generate_gaussian(Ncmp,vel,initA)
plot,vel,spec,xr=vrange,/xsty,psym=10,xtitle='!6Vlsr km/s',$
ytitle='!6Tr* K',title='Least square fitted data',yr=trange,/ysty
oplot,vel,ff,color=120
FOR j=0,Ncmp-1 DO BEGIN
   f=generate_gaussian(1,vel,initA(3*j:3*j+2))
   oplot,vel,f,color=100
ENDFOR
ddx=(vrange(1)-vrange(0))/20.
ddy=(trange(1)-trange(0))/20.
xpos=vrange(0)+ddx
ypos=trange(1)-ddy
xyouts,xpos,ypos,'Tr*        Vlsr        dV',/DATA,color=100,SIZE=1.5
FOR j=0,Ncmp-1 DO BEGIN
   ypos=ypos-ddy
   FOR k=0,3-1 DO BEGIN
      charac=strcompress(string(initA(3*j+k)),/remove_all)
      charac=strmid(charac,0,6)
      xyouts,xpos,ypos,charac,/DATA,color=100,SIZE=1.5
      xpos=xpos+ddx*3.
   ENDFOR
  xpos=xpos-ddx*9.
ENDFOR

endingline:
return,yfit

END
