FUNCTION generate_gaussian,Ncmp,X,A
;+
;PURPOSE:
;	Generate a function with N gaussian components.
;CALLING SEQUENCE:
;	Result=generate_gaussian(Ncmp,X,A)
;INPUT:
;	Ncmp	=	Scalor. Number of gaussian components.
;	X	=	1D array for the X axis.
;	A	=	1D array for the gaussian coefficients.
;			 (See the following example)
;OUTPUT:
;	Result	=	1D array with the given gaussian components.
;EXAMPLE:
;	for Ncmp=2, X=findgen(100), and A=(10.,20.,5.,5.,60,10),
;	Do "Result=generate_gaussian(Ncmp,X,A)". See what is given
;	back in "Result".
;MODIFICATION HIASTORY:
;	Written by K. Dobashi Feb-97
;
;-

on_error,2	;return to caller

; Check N_param================
if N_params() LT 3 then begin	
      print,'Generate gaussian functions'	
      print,"Usage - Result=generate_gaussian(Ncmp,X,A)"
      print,'Input - Ncmp(number of gaussian components),X(x data),'
      print,'        A(coefficients)'
      print,'Output - Result,A (1D array)'
      print,'A(i) - y=A(3*i)exp(-((x-A(3*i+1))/sqrt(2)/A(3*i+2))^2)'
      print,'Result with be the sum of y defined above.'
     goto,sortie
  endif
;start==============================
Ngau=Ncmp	;number of gaussian comp.
F=[x] & F(*)=0.
FOR i=0,Ngau-1 DO BEGIN
   ind=where(abs(X-A(3*i+1)) LT 10*A(3*i+2),count)
   IF count NE 0 THEN BEGIN
      F(ind)=F(ind)+A(3*i)*exp(-((X(ind)-A(3*i+1))/A(3*i+2)/sqrt(2.))^2)
   ENDIF
ENDFOR

return,F
sortie:
end

