FUNCTION my_bilinear3 , array, x, y ,silent=silent, no_remove_indef=no_remove_indef

;+
; NAME:
;        my_bilinear2
; PURPOSE:
;       same as bilinear of user library but can deal with big arrays
; CATEGORY:
;       Coordinates conversions
; CALLING SEQUENCE: 
;	my_bilinear , array, x, y
; INPUTS:
;	array = array (2D) in which interpolation is to be made
;	x, y = 2D array of x and y values (must be of same size)
; OPTIONAL INPUT PARAMETERS:
; OUTPUTS:
;	function output=interpolated array (same size as x and y)
; OPTIONAL OUTPUT PARAMETERS:
; COMMON BLOCKS:
; SIDE EFFECTS:
;	x and y values not in the target image (outside array)
;	are changed to 0 or -1
; RESTRICTIONS:
; PROCEDURE:
;	BILINEAR (users library)
;	indef values are difficult to handle. Here values in the
;	interpolated values which values are smaller than the smaller
;	defined value in the original array are set to indef. This works
;	for a highly <0 indef (e.g indef=-32768.) and generally >0 image
; MODIFICATION HISTORY:
;      14-Aug-1992, Written by Jean-Philippe Bernard, Nagoya Univ.
;-

;---------------------------------------------------------------
; parameter check
;---------------------------------------------------------------
IF N_PARAMS(0) LT 3 THEN BEGIN
  PRINT,'Calling sequence: res=my_bilinear3(array, x, y)'
  PRINT,'K words:              [/silent]'
  PRINT,'Arrays are 1D'
  GOTO, closing
ENDIF

step_size=100

six=SIZE(x)
siy=SIZE(y)
siA=SIZE(array)
IF six(0) NE 1 OR siy(0) NE 1 THEN BEGIN
  print,'x and y should be 1D arrays'
  goto,closing
ENDIF
IF six(1) NE siy(1) OR six(2) NE siy(2) THEN BEGIN
  print,'Incompatible x and y array dimensions'
  goto,closing
ENDIF
IF siA(0) NE 2 THEN BEGIN
  print,'array should be a 2D array'
  goto,closing
ENDIF


;---------------------------------------------------------------
; calculations
;---------------------------------------------------------------

Npt=six(1)
Nax=siA(1)
Nay=siA(2)

output=fltarr(Npt) & output(*,*)=!indef

min_array=min(array)
ind=where(array ne !indef,count)
IF count NE 0 THEN min_array=min(array(ind))

indbad=where(x LT 0 OR x GT nax-1 OR y LT 0 or y GT nay-1,countbad)
inter_percent=1.*(Nax*Nay-countbad)/Nax/Nay*100.
IF not keyword_set(silent) THEN BEGIN
  print,'Images intersection=',inter_percent,' %'
ENDIF

;stop

ind=where(x GE 0 AND x LE Nax-1 AND y GE 0 AND y LE Nay-1,count)
IF count NE 0 THEN BEGIN
  FOR j=0L,count-1,step_size DO BEGIN
    from=j & to=j+step_size-1
    to=to<(Npt-1)
    xx=x(from:to) & yy=y(from:to)
;    xx=reform(xx,1,step_size) & yy=reform(yy,1,step_size)
    xx=reform(xx,1,(size(xx))(1)) & yy=reform(yy,1,(size(yy))(1))
    truc=bilinear(array,xx,yy)
    output(from:to)=truc(0,*)
  ENDFOR
ENDIF

;remove values affected by indef values (for highly <0 indef and generaly>0 im)
IF not keyword_set(no_remove_indef) THEN BEGIN
  ind=where(output LT min_array,count)
  IF count NE 0 THEN output(ind)=!indef
ENDIF

return,output
CLOSING:
END
