FUNCTION project2,h1,im1,h2,silent=silent, $
		 save_it=save_it,restore_it=restore_it, $
		 previous_lut=previous_lut,lbl=lbl,round=round

;+
; NAME:
;       project2
; CALLING SEQUENCE:
;	im2=project2(h1,im1,h2)
; PURPOSE:
;	reprojects im1,h1 into the projection specified by h2
; INPUTS:
;       im1,h1 = image, header to be reprojected
;       h2 = header of the output image (must have a different proj as h1)
; OPTIONAL INPUT:
;	save_it=to save the project parameters
;	restore_it=to restore project parameters
;	previous_lut=to use the previous projection
; OUTPUTS:
;       im2 = result of the reprojection
; SUBROUTINE AND PROCEDURE USED
;       sxpar,create_coo2,coo2pix2,my_bilinear, coo2coo
; SIDE EFFECTS:
; RESTRICTIONS:
;       Should take coord,epoch,Naxis,.... into account.
; MODIFICATION HISTORY:
;       written J.P. Bernard @ IPAC Jan 31 1995 
;-

;ON_ERROR,2

@proj_common.com
@lut_proj.com

IF N_PARAMS(0) LT 3 THEN BEGIN
  Print,'Calling Sequence: im2=project2(h1,im1,h2)'
  print,'Key words       :     save_it=,restore_it=,/silent,/previous_lut,/lbl'
  print,'                      /round'
  goto,ending
ENDIF

indef=-32768.
IF not keyword_set(silent) THEN print,'Starting @ ',systime(0)
IF keyword_set(restore_it) THEN BEGIN
  IF not keyword_set(silent) THEN print,'Restoring ',restore_it
  restore,restore_it
  goto,interp
ENDIF
IF keyword_set(previous_lut) THEN BEGIN
  x=last_x_proj & y=last_y_proj
  h2=last_headout
  goto,interp
ENDIF
;=== Make the projection table, eventally saving it
create_coo2,h2,c1,c2,silent=silent,lbl=lbl
coord2=coord & Eq2=float(equinox)
h2pix,h1,silent=silent
coord1=coord & Eq1=float(equinox)
;If necessary, precess coordinates to match h1 header if same coords
;Otherwise precess to 1950, because convcoord works for 1950.
;For 'RA', convert to equinox eq1, or 1950 if coord conv necessary
to1950=0
IF coord2 EQ 'RA' THEN BEGIN
  IF coord1 EQ 'RA' THEN BEGIN
    ind=where(c1 NE indef) & cc1=c1(ind) & cc2=c2(ind)
;    stop
    si=size(cc1) & Ncc1=si(1)
    FOR i=0l,Ncc1-1 DO BEGIN
      ccc1=cc1(i) & ccc2=cc2(i)
      precess,ccc1,ccc2,Eq2,Eq1
      cc1(i)=ccc1 & cc2(i)=ccc2
    ENDFOR
    c1(ind)=cc1 & c2(ind)=cc2
  ENDIF ELSE BEGIN
    ind=where(c1 NE indef) & cc1=c1(ind) & cc2=c2(ind)
    si=size(cc1) & Ncc1=si(1)
    FOR i=0l,Ncc1-1 DO BEGIN
      ccc1=cc1(i) & ccc2=cc2(i)
      precess,ccc1,ccc2,Eq2,1950.
      cc1(i)=ccc1 & cc2(i)=ccc2
    ENDFOR
    c1(ind)=cc1 & c2(ind)=cc2
    to1950=1
  ENDELSE
ENDIF
;For 'ELON', convert to 'RA', precess to 1950, bring back to elon
IF coord2 EQ 'ELON' THEN BEGIN
  outco=coo2co('RA')
  inco=coo2co(coord2)
  ind=where(c1 NE indef,count)
  vec=fltarr(count,2) & vec(*,0)=c1(ind) & vec(*,1)=c2(ind)
  vec=my_coorconv(vec,infmt='l',outfmt='l',inco=inco,outco=outco)
  cc1=vec(*,0) & cc2=vec(*,1)
  si=size(cc1) & Ncc1=si(1)
  FOR i=0l,Ncc1-1 DO BEGIN
    ccc1=cc1(i) & ccc2=cc2(i)
    precess,ccc1,ccc2,Eq2,1950.
    cc1(i)=ccc1 & cc2(i)=ccc2
  ENDFOR
  outco=coo2co(coord2)
  inco=coo2co('RA')
  vec(*,0)=cc1 & vec(*,1)=cc2
  vec=my_coorconv(vec,infmt='l',outfmt='l',inco=inco,outco=outco)
  c1(ind)=vec(*,0) & c2(ind)=vec(*,1)
  to1950=1
ENDIF
;======== Convert coordinates from coord2 to coord1
inco=coo2co(coord2)
outco=coo2co(coord1)
IF inco NE outco THEN BEGIN
  ind=where(c1 NE indef,count)
  vec=fltarr(count,2) & vec(*,0)=c1(ind) & vec(*,1)=c2(ind)
  vec=my_coorconv(vec,infmt='l',outfmt='l',inco=inco,outco=outco)
  c1(ind)=vec(*,0) & c2(ind)=vec(*,1)
ENDIF

;precess to Eq1, if necessary
IF coord1 EQ 'RA' AND to1950 EQ 1 THEN BEGIN
  IF eq1 NE 1950 THEN BEGIN
    ind=where(c1 NE indef) & cc1=c1(ind) & cc2=c2(ind)
    si=size(cc1) & Ncc1=si(1)
    FOR i=0l,Ncc1-1 DO BEGIN
      ccc1=cc1(i) & ccc2=cc2(i)
      precess,ccc1,ccc2,1950.,eq1
      cc1(i)=ccc1 & cc2(i)=ccc2
    ENDFOR  
    c1(ind)=cc1 & c2(ind)=cc2
  ENDIF
ENDIF
IF coord1 EQ 'ELON' THEN BEGIN
  IF eq1 NE 1950 THEN BEGIN
    outco=coo2co('RA')
    inco=coo2co(coord1)
    ind=where(c1 NE indef,count)
    vec=fltarr(count,2) & vec(*,0)=c1(ind) & vec(*,1)=c2(ind)
    vec=my_coorconv(vec,infmt='l',outfmt='l',inco=inco,outco=outco)
    cc1=vec(*,0) & cc2=vec(*,1)
    precess,cc1,cc2,1950.,Eq1
    outco=coo2co(coord1)
    inco=coo2co('RA')
    vec(*,0)=cc1(ind) & vec(*,1)=cc2(ind)
    vec=my_coorconv(vec,infmt='l',outfmt='l',inco=inco,outco=outco)
    c1(ind)=vec(*,0) & c2(ind)=vec(*,1)
  ENDIF
ENDIF

coo2pix2,c1,c2,x,y,silent=silent,lbl=lbl
;stop
IF keyword_set(round) THEN BEGIN
  x=round(x) & y=round(y)
ENDIF

last_x_proj=x & last_y_proj=y & last_headout=h2
;mk_proj,h1,h2,x,y,save_it=save_it,silent=silent
IF keyword_set(save_it) THEN BEGIN
  IF not keyword_set(silent) THEN print,'saving ',save_it
  save,x,y,file=save_it
ENDIF

;=== interpolate into im1
interp:
IF not keyword_set(silent) THEN print,'Interpolating im2 into im1'
im2=my_bilinear2(im1,x,y,silent=silent)

IF not keyword_set(silent) THEN print,'Ending @ ',systime(0)

RETURN,im2
ending:

END
