FUNCTION rebin_image,im,h,hout,ffact,sample=sample,median=median

;+
; NAME:
;       rebin_image
; PURPOSE:
;       scales a fits image by a given scale factor
;       and update header info.
; CATEGORY:
;       image_processing
; CALLING SEQUENCE:
;       im=rebin_image(image,header,hout,ffact[,/sample][,/median])
; INPUTS:
;       image          = input image (2D)
;       header         = header for image
;       ffact          = rebin factor
; OPTIONAL INPUTS:
;       sample         = to keep pixel information
;       median         = ?
; KEYWORD PARAMETERS:
; OUTPUTS:
;       hout           = modified header (for im)
;       im             = rebined image
; OPTIONAL OUTPUTS:
; COMMON BLOCKS:
;       None
; SIDE EFFECTS:
; RESTRICTIONS:
; PROCEDURE:
; EXAMPLE:
; MODIFICATION HISTORY:
;       written JPB 09-08-92
;       Corrected JPB 07-03-02 for a bug in crpix1,crpix2 values
;-

IF N_PARAMS(0) LT 4 THEN BEGIN
    print,'Calling Sequence: im=rebin_image(image,header,hout,ffact)'
    print,'Accepted Key-words: /sample,/median'
    GOTO,sortie
ENDIF


si=size(im) & Nx=si(1) & Ny=si(2)
IF ffact GT 0 THEN BEGIN
  fact=1.*ffact
  IF keyword_set(sample) THEN BEGIN
    imout=rebin(im,Nx*fact,Ny*fact,sample=sample)
  ENDIF ELSE BEGIN
    imout=congrid(im,Nx*fact,Ny*fact,cubic=-0.5,/minus_one)
  ENDELSE
  hout=h
  sxaddpar,hout,'NAXIS1',Nx*fact
  sxaddpar,hout,'NAXIS2',Ny*fact
  sxaddpar,hout,'CDELT1',sxpar(h,'CDELT1')/fact
  sxaddpar,hout,'CDELT2',sxpar(h,'CDELT2')/fact
  ; sxaddpar,hout,'CRPIX1',sxpar(h,'CRPIX1')*fact
  ; sxaddpar,hout,'CRPIX2',sxpar(h,'CRPIX2')*fact
  sxaddpar,hout,'CRPIX1',sxpar(h,'CRPIX1')*fact-fact/2.+0.5
  sxaddpar,hout,'CRPIX2',sxpar(h,'CRPIX2')*fact-fact/2.+0.5
ENDIF ELSE BEGIN
  fact=-1./ffact
  ffac=-ffact
  imout=fltarr(Nx*fact,Ny*fact)
  FOR i=0,Nx*fact-1 DO BEGIN 
    FOR j=0,Ny*fact-1 DO BEGIN
      ifr=i*ffac & ito=(i+1)*ffac-1
      jfr=j*ffac & jto=(j+1)*ffac-1
      subim=im(ifr:ito,jfr:jto)
      ind=where(subim NE !indef,count)
      IF count NE 0 THEN BEGIN
        IF keyword_set(median) THEN BEGIN
          imout(i,j)=median(subim(ind))
        ENDIF ELSE BEGIN
          imout(i,j)=avg(subim(ind))
        ENDELSE
      ENDIF
    ENDFOR
  ENDFOR
  hout=h
  sxaddpar,hout,'NAXIS1',Nx*fact
  sxaddpar,hout,'NAXIS2',Ny*fact
  sxaddpar,hout,'CDELT1',sxpar(h,'CDELT1')*ffac
  sxaddpar,hout,'CDELT2',sxpar(h,'CDELT2')*ffac
  ;sxaddpar,hout,'CRPIX1',sxpar(h,'CRPIX1')/ffac
  ;sxaddpar,hout,'CRPIX2',sxpar(h,'CRPIX2')/ffac
  sxaddpar,hout,'CRPIX1',(sxpar(h,'CRPIX1')+1./ffac/2.-0.5)/ffac
  sxaddpar,hout,'CRPIX2',(sxpar(h,'CRPIX2')+1./ffac/2.-0.5)/ffac
ENDELSE

RETURN,imout

sortie:

END
