PRO mac_cp,file1,file2,empty_trash=empty_trash,script=script,activate=activate

;+
; NAME:
;	   mac_cp
; PURPOSE:
;       (almost) equivalent to the unix "cp" command for the Mac
; CATEGORY:
;       Mac routines
; CALLING SEQUENCE:
;	   mac_cp,file1,file2,script=script,activate=activate
; INPUTS:
;	   file1    = file to be copied, including its full path. Can contain *
;       file2    = directory toward which the copy is to be made
; OPTIONAL INPUT PARAMETERS:
;       None
; OUTPUTS:
;       None
; OPTIONAL OUTPUT PARAMETERS:
;	   script  = the apple script used
; ACCEPTED KEY-WORDS:
;       activate = if set, the finder is activated.
; COMMON BLOCKS:
;       None
; SIDE EFFECTS:
;       A copy is made. The original file is not removed
; RESTRICTIONS:
;       Works only with IDL on a Macintosh
; PROCEDURE:
; EXAMPLES
;       mac_cp,'ISSA_0000:IMAGES:I*B4H0.FIT','Xcaret6G:Data:IRAS:',/activate    
; MODIFICATION HISTORY:
;	Written by J.P. Bernard Dec 16 1999
;-

;---------------------------------------------------------------
; parameter check
;---------------------------------------------------------------
IF N_PARAMS(0) LT 2 THEN BEGIN
  PRINT,'Calling Sequence: '
  PRINT,'mac_cp,file1,file2[,script=script][,/activate]'
  PRINT,'         file1 may incorporate *'
  GOTO, sortie
ENDIF

files1=findfile(file1,count=c1)
IF c1 EQ 0 THEN BEGIN
  message,'File:'+file1+' Not found',/info
  goto,sortie
ENDIF ELSE BEGIN
  message,'Found '+strtrim(c1,2)+' Files',/info
ENDELSE

pos=strposmulti(file2,':',c)
dir2=strmid(file2,0,pos(c-1))
ffile=findfile(dir2+'*',count=c)
IF c NE 0 THEN BEGIN
  ind=where(ffile EQ dir2,cc)
ENDIF ELSE BEGIN
  message,'Creating directory:'+dir2,/info
  mac_mkdir,dir2
ENDELSE

quote='"'
script=['tell application "Finder"']
IF keyword_set(activate) THEN script=[script,'activate']
file=findfile(file2,count=count)
IF count NE 0 THEN BEGIN
  str='select file '+quote+file2+quote
  script=[script,str]
 script=[script,'delete selection']
ENDIF
IF c1 EQ 1 THEN BEGIN
  str='select file '+quote+files1(0)+quote
ENDIF ELSE BEGIN
  str='select {'
  FOR j=0,c1-1 DO BEGIN
    str=str+'file '+quote+files1(j)+quote
    IF j EQ c1-1 THEN str=str+'}' ELSE str=str+','
  ENDFOR
ENDELSE
script=[script,str]
pos=strposmulti(file2,':',c)
dir2=strmid(file2,0,pos(c-1))
str='copy selection to folder '+quote+dir2+quote
script=[script,str]

script=[script,'end tell']

;stop

do_apple_script,script

sortie:

END
